#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_adapter_config_policy
short_description: Adapter Configuration Policy for Cisco Intersight
description:
  - Manages Adapter Configuration Policy on Cisco Intersight.
  - This policy allows you to configure adapter settings such as LLDP, FIP, Port Channel, and FEC mode.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Adapter Configuration Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Adapter Configuration Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
    default: []
  settings:
    description:
      - List of adapter configurations.
    type: list
    elements: dict
    suboptions:
      slot_id:
        description:
          - PCIe slot where the VIC adapter is installed.
          - Supported values are (1-15) and MLOM.
        type: str
        required: true
      enable_lldp:
        description:
          - Status of LLDP protocol on the adapter interfaces.
        type: bool
        default: true
      enable_fip:
        description:
          - Status of FIP protocol on the adapter interfaces.
        type: bool
        default: true
      enable_port_channel:
        description:
          - Status of Port Channel on the adapter interfaces.
          - When enabled, two vNICs and two vHBAs are available.
          - When disabled, four vNICs and four vHBAs are available.
          - Disabling port channel reboots the server.
          - Supported only for Cisco VIC 1455/1457 adapters.
        type: bool
        default: true
      dce_interface_1_fec_mode:
        description:
          - Forward Error Correction (FEC) mode setting for DCE interface 1 (Interface ID 0).
          - Supported only for Cisco VIC 14xx adapters.
          - FEC mode 'cl74' is unsupported for Cisco VIC 1495/1497.
        type: str
        choices: ['cl91', 'cl74', 'off']
        default: cl91
      dce_interface_2_fec_mode:
        description:
          - Forward Error Correction (FEC) mode setting for DCE interface 2 (Interface ID 1).
        type: str
        choices: ['cl91', 'cl74', 'off']
        default: cl91
      dce_interface_3_fec_mode:
        description:
          - Forward Error Correction (FEC) mode setting for DCE interface 3 (Interface ID 2).
        type: str
        choices: ['cl91', 'cl74', 'off']
        default: cl91
      dce_interface_4_fec_mode:
        description:
          - Forward Error Correction (FEC) mode setting for DCE interface 4 (Interface ID 3).
        type: str
        choices: ['cl91', 'cl74', 'off']
        default: cl91
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create Adapter Config Policy
  cisco.intersight.intersight_adapter_config_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "adapter-config-policy"
    description: "Policy for VIC 1455"
    settings:
      - slot_id: "2"
        enable_lldp: true
        enable_fip: true
        enable_port_channel: true
        dce_interface_1_fec_mode: "cl91"
        dce_interface_2_fec_mode: "cl91"
        dce_interface_3_fec_mode: "cl91"
        dce_interface_4_fec_mode: "off"
    state: present

- name: Delete Adapter Config Policy
  cisco.intersight.intersight_adapter_config_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "adapter-config-policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample: {
    "Name": "adapter-config-policy",
    "ObjectType": "adapter.ConfigPolicy",
    "Settings": [
      {
        "SlotId": "2",
        "EthSettings": {
          "LldpEnabled": true
        },
        "FcSettings": {
          "FipEnabled": true
        },
        "PortChannelSettings": {
          "Enabled": true
        },
        "DceInterfaceSettings": [
          {
            "FecMode": "cl91",
            "InterfaceId": 0
          }
        ]
      }
    ]
  }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def format_fec_mode(mode):
    """
    Format FEC mode string for API.
    'off' should be 'Off'.
    """
    if mode and mode.lower() == 'off':
        return 'Off'
    return mode


def build_settings_config(settings):
    """
    Build Settings configuration for API body.
    """
    settings_list = []
    for setting in settings:
        config = {
            'ObjectType': 'adapter.AdapterConfig',
            'SlotId': setting['slot_id'],
            'EthSettings': {
                'LldpEnabled': setting['enable_lldp'],
                'ObjectType': 'adapter.EthSettings'
            },
            'FcSettings': {
                'FipEnabled': setting['enable_fip'],
                'ObjectType': 'adapter.FcSettings'
            },
            'PortChannelSettings': {
                'Enabled': setting['enable_port_channel'],
                'ObjectType': 'adapter.PortChannelSettings'
            },
            'PhysicalNicModeSettings': {
                'PhyNicEnabled': False,
                'ObjectType': 'adapter.PhysicalNicModeSettings'
            }
        }

        dce_settings = []
        # Interface 1
        dce_settings.append({
            'ObjectType': 'adapter.DceInterfaceSettings',
            'InterfaceId': 0,
            'FecMode': format_fec_mode(setting['dce_interface_1_fec_mode'])
        })
        # Interface 2
        dce_settings.append({
            'ObjectType': 'adapter.DceInterfaceSettings',
            'InterfaceId': 1,
            'FecMode': format_fec_mode(setting['dce_interface_2_fec_mode'])
        })
        # Interface 3
        dce_settings.append({
            'ObjectType': 'adapter.DceInterfaceSettings',
            'InterfaceId': 2,
            'FecMode': format_fec_mode(setting['dce_interface_3_fec_mode'])
        })
        # Interface 4
        dce_settings.append({
            'ObjectType': 'adapter.DceInterfaceSettings',
            'InterfaceId': 3,
            'FecMode': format_fec_mode(setting['dce_interface_4_fec_mode'])
        })

        config['DceInterfaceSettings'] = dce_settings
        settings_list.append(config)

    return settings_list


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict', default=[]),
        settings=dict(
            type='list',
            elements='dict',
            options=dict(
                slot_id=dict(type='str', required=True),
                enable_lldp=dict(type='bool', default=True),
                enable_fip=dict(type='bool', default=True),
                enable_port_channel=dict(type='bool', default=True),
                dce_interface_1_fec_mode=dict(type='str', choices=['cl91', 'cl74', 'off'], default='cl91'),
                dce_interface_2_fec_mode=dict(type='str', choices=['cl91', 'cl74', 'off'], default='cl91'),
                dce_interface_3_fec_mode=dict(type='str', choices=['cl91', 'cl74', 'off'], default='cl91'),
                dce_interface_4_fec_mode=dict(type='str', choices=['cl91', 'cl74', 'off'], default='cl91'),
            )
        )
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    intersight.api_body = {
        'Organization': {
            'Name': module.params['organization'],
        },
        'Name': module.params['name'],
        'ObjectType': 'adapter.ConfigPolicy'
    }

    if module.params['state'] == 'present':
        intersight.set_tags_and_description()

        if module.params.get('settings'):
            intersight.api_body['Settings'] = build_settings_config(module.params['settings'])

    resource_path = '/adapter/ConfigPolicies'
    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# Copyright (c) 2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: dict
short_description: Convert a list of tuples into a dictionary
version_added: 3.0.0
author: Felix Fontein (@felixfontein)
description:
  - Convert a list of tuples into a dictionary. This is a filter version of the C(dict) function.
options:
  _input:
    description: A list of tuples (with exactly two elements).
    type: list
    elements: tuple
    required: true
"""

EXAMPLES = r"""
- name: Convert list of tuples into dictionary
  ansible.builtin.set_fact:
    dictionary: "{{ [[1, 2], ['a', 'b']] | community.general.dict }}"
    # Result is {1: 2, 'a': 'b'}

- name: Create a list of dictionaries with map and the community.general.dict filter
  ansible.builtin.debug:
    msg: >-
      {{ values | map('zip', ['k1', 'k2', 'k3'])
                | map('map', 'reverse')
                | map('community.general.dict') }}
  vars:
    values:
      - - foo
        - 23
        - a
      - - bar
        - 42
        - b
  # Produces the following list of dictionaries:
  #   {
  #     "k1": "foo",
  #     "k2": 23,
  #     "k3": "a"
  #   },
  #   {
  #     "k1": "bar",
  #     "k2": 42,
  #     "k3": "b"
  #   }
"""

RETURN = r"""
_value:
  description: A dictionary with the provided key-value pairs.
  type: dictionary
"""


def dict_filter(sequence):
    """Convert a list of tuples to a dictionary.

    Example: ``[[1, 2], ['a', 'b']] | community.general.dict`` results in ``{1: 2, 'a': 'b'}``
    """
    return dict(sequence)


class FilterModule:
    """Ansible jinja2 filters"""

    def filters(self):
        return {
            "dict": dict_filter,
        }



# Copyright (c), Google Inc, 2017
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import re
import time
import traceback
import typing as t
from http import HTTPStatus

THIRD_LIBRARIES_IMP_ERR = None
try:
    from keystoneauth1 import session
    from keystoneauth1.adapter import Adapter
    from keystoneauth1.identity import v3

    HAS_THIRD_LIBRARIES = True
except ImportError:
    THIRD_LIBRARIES_IMP_ERR = traceback.format_exc()
    HAS_THIRD_LIBRARIES = False

from ansible.module_utils.basic import AnsibleModule, env_fallback, missing_required_lib
from ansible.module_utils.common.text.converters import to_text


class HwcModuleException(Exception):
    def __init__(self, message: str) -> None:
        super().__init__()

        self._message = message

    def __str__(self) -> str:
        return f"[HwcClientException] message={self._message}"


class HwcClientException(Exception):
    def __init__(self, code: int, message: str) -> None:
        super().__init__()

        self._code = code
        self._message = message

    def __str__(self) -> str:
        msg = f" code={self._code}," if self._code != 0 else ""
        return f"[HwcClientException]{msg} message={self._message}"


class HwcClientException404(HwcClientException):
    def __init__(self, message: str) -> None:
        super().__init__(404, message)

    def __str__(self) -> str:
        return f"[HwcClientException404] message={self._message}"


def session_method_wrapper(f):
    def _wrap(self, url: str, *args, **kwargs):
        try:
            url = self.endpoint + url
            r = f(self, url, *args, **kwargs)
        except Exception as ex:
            raise HwcClientException(0, f"Sending request failed, error={ex}") from ex

        result = None
        if r.content:
            try:
                result = r.json()
            except Exception as ex:
                raise HwcClientException(0, f"Parsing response to json failed, error: {ex}") from ex

        code = r.status_code
        if not (HTTPStatus.OK <= code < HTTPStatus.MULTIPLE_CHOICES):  # not 2xx codes
            msg = ""
            for i in ["message", "error.message"]:
                try:
                    msg = navigate_value(result, i)
                    break
                except Exception:
                    pass
            else:
                msg = str(result)

            if code == HTTPStatus.NOT_FOUND:
                raise HwcClientException404(msg)

            raise HwcClientException(code, msg)

        return result

    return _wrap


class _ServiceClient:
    def __init__(self, client, endpoint: str, product):
        self._client = client
        self._endpoint = endpoint
        self._default_header = {
            "User-Agent": f"Huawei-Ansible-MM-{product}",
            "Accept": "application/json",
        }

    @property
    def endpoint(self) -> str:
        return self._endpoint

    @endpoint.setter
    def endpoint(self, e: str) -> None:
        self._endpoint = e

    @session_method_wrapper
    def get(self, url: str, body=None, header: dict[str, t.Any] | None = None, timeout=None):
        return self._client.get(url, json=body, timeout=timeout, headers=self._header(header))

    @session_method_wrapper
    def post(self, url: str, body=None, header: dict[str, t.Any] | None = None, timeout=None):
        return self._client.post(url, json=body, timeout=timeout, headers=self._header(header))

    @session_method_wrapper
    def delete(self, url: str, body=None, header: dict[str, t.Any] | None = None, timeout=None):
        return self._client.delete(url, json=body, timeout=timeout, headers=self._header(header))

    @session_method_wrapper
    def put(self, url: str, body=None, header: dict[str, t.Any] | None = None, timeout=None):
        return self._client.put(url, json=body, timeout=timeout, headers=self._header(header))

    def _header(self, header: dict[str, t.Any] | None) -> dict[str, t.Any]:
        if header and isinstance(header, dict):
            for k, v in self._default_header.items():
                if k not in header:
                    header[k] = v
        else:
            header = self._default_header

        return header


class Config:
    def __init__(self, module: AnsibleModule, product) -> None:
        self._project_client = None
        self._domain_client = None
        self._module = module
        self._product = product
        self._endpoints: dict[str, t.Any] = {}

        self._validate()
        self._gen_provider_client()

    @property
    def module(self) -> AnsibleModule:
        return self._module

    def client(self, region, service_type, service_level):
        c = self._project_client
        if service_level == "domain":
            c = self._domain_client

        e = self._get_service_endpoint(c, service_type, region)

        return _ServiceClient(c, e, self._product)

    def _gen_provider_client(self):
        m = self._module
        p = {
            "auth_url": m.params["identity_endpoint"],
            "password": m.params["password"],
            "username": m.params["user"],
            "project_name": m.params["project"],
            "user_domain_name": m.params["domain"],
            "reauthenticate": True,
        }

        self._project_client = Adapter(session.Session(auth=v3.Password(**p)), raise_exc=False)

        p.pop("project_name")
        self._domain_client = Adapter(session.Session(auth=v3.Password(**p)), raise_exc=False)

    def _get_service_endpoint(self, client, service_type, region):
        k = f"{service_type}.{region if region else ''}"

        if k in self._endpoints:
            return self._endpoints.get(k)

        url = None
        try:
            url = client.get_endpoint(service_type=service_type, region_name=region, interface="public")
        except Exception as ex:
            raise HwcClientException(0, f"Getting endpoint failed, error={ex}") from ex

        if url == "":
            raise HwcClientException(0, f"Cannot find the endpoint for {service_type}")

        if url[-1] != "/":
            url += "/"

        self._endpoints[k] = url
        return url

    def _validate(self):
        if not HAS_THIRD_LIBRARIES:
            self.module.fail_json(msg=missing_required_lib("keystoneauth1"), exception=THIRD_LIBRARIES_IMP_ERR)


class HwcModule(AnsibleModule):
    def __init__(self, *args, **kwargs):
        arg_spec = kwargs.setdefault("argument_spec", {})

        arg_spec.update(
            dict(
                identity_endpoint=dict(
                    required=True,
                    type="str",
                    fallback=(env_fallback, ["ANSIBLE_HWC_IDENTITY_ENDPOINT"]),
                ),
                user=dict(
                    required=True,
                    type="str",
                    fallback=(env_fallback, ["ANSIBLE_HWC_USER"]),
                ),
                password=dict(
                    required=True,
                    type="str",
                    no_log=True,
                    fallback=(env_fallback, ["ANSIBLE_HWC_PASSWORD"]),
                ),
                domain=dict(
                    required=True,
                    type="str",
                    fallback=(env_fallback, ["ANSIBLE_HWC_DOMAIN"]),
                ),
                project=dict(
                    required=True,
                    type="str",
                    fallback=(env_fallback, ["ANSIBLE_HWC_PROJECT"]),
                ),
                region=dict(
                    type="str",
                    fallback=(env_fallback, ["ANSIBLE_HWC_REGION"]),
                ),
                id=dict(type="str"),
            )
        )

        super().__init__(*args, **kwargs)


class _DictComparison:
    """This class takes in two dictionaries `a` and `b`.
    These are dictionaries of arbitrary depth, but made up of standard
    Python types only.
    This differ will compare all values in `a` to those in `b`.
    If value in `a` is None, always returns True, indicating
    this value is no need to compare.
    Note: On all lists, order does matter.
    """

    def __init__(self, request):
        self.request = request

    def __eq__(self, other):
        return self._compare_dicts(self.request, other.request)

    def __ne__(self, other):
        return not self.__eq__(other)

    def _compare_dicts(self, dict1, dict2):
        if dict1 is None:
            return True

        if set(dict1.keys()) != set(dict2.keys()):
            return False

        return all(self._compare_value(dict1[k], dict2[k]) for k in dict1)

    def _compare_lists(self, list1, list2):
        """Takes in two lists and compares them."""
        if list1 is None:
            return True

        if len(list1) != len(list2):
            return False

        return all(self._compare_value(l1, l2) for l1, l2 in zip(list1, list2))

    def _compare_value(self, value1, value2):
        """
        return: True: value1 is same as value2, otherwise False.
        """
        if value1 is None:
            return True

        if not (value1 and value2):
            return (not value1) and (not value2)

        # Can assume non-None types at this point.
        if isinstance(value1, list) and isinstance(value2, list):
            return self._compare_lists(value1, value2)

        elif isinstance(value1, dict) and isinstance(value2, dict):
            return self._compare_dicts(value1, value2)

        # Always use to_text values to avoid unicode issues.
        return to_text(value1, errors="surrogate_or_strict") == to_text(value2, errors="surrogate_or_strict")


def wait_to_finish(target, pending, refresh, timeout, min_interval=1, delay=3):
    is_last_time = False
    not_found_times = 0
    wait = 0

    time.sleep(delay)

    end = time.time() + timeout
    while not is_last_time:
        if time.time() > end:
            is_last_time = True

        obj, status = refresh()

        if obj is None:
            not_found_times += 1

            if not_found_times > 10:
                raise HwcModuleException(f"not found the object for {not_found_times} times")
        else:
            not_found_times = 0

            if status in target:
                return obj

            if pending and status not in pending:
                raise HwcModuleException(f"unexpected status({status}) occurred")

        if not is_last_time:
            wait *= 2
            if wait < min_interval:
                wait = min_interval
            elif wait > 10:
                wait = 10

            time.sleep(wait)

    raise HwcModuleException(f"async wait timeout after {timeout} seconds")


def navigate_value(data, index, array_index=None):
    if array_index and (not isinstance(array_index, dict)):
        raise HwcModuleException("array_index must be dict")

    d = data
    for n in range(len(index)):
        if d is None:
            return None

        if not isinstance(d, dict):
            raise HwcModuleException("can't navigate value from a non-dict object")

        i = index[n]
        if i not in d:
            raise HwcModuleException(f"navigate value failed: key({i}) is not exist in dict")
        d = d[i]

        if not array_index:
            continue

        k = ".".join(index[: (n + 1)])
        if k not in array_index:
            continue

        if d is None:
            return None

        if not isinstance(d, list):
            raise HwcModuleException("can't navigate value from a non-list object")

        j = array_index.get(k)
        if j >= len(d):
            raise HwcModuleException("navigate value failed: the index is out of list")
        d = d[j]

    return d


def build_path(module: AnsibleModule, path, kv=None):
    if kv is None:
        kv = dict()

    v = {}
    for p in re.findall(r"{[^/]*}", path):
        n = p[1:][:-1]

        if n in kv:
            v[n] = str(kv[n])

        else:
            if n in module.params:
                v[n] = str(module.params.get(n))
            else:
                v[n] = ""

    return path.format(**v)


def get_region(module: AnsibleModule):
    if module.params["region"]:
        return module.params["region"]

    return module.params["project"].split("_")[0]


def is_empty_value(v):
    return not v


def are_different_dicts(dict1, dict2):
    return _DictComparison(dict1) != _DictComparison(dict2)



# -*- coding: utf-8 -*-

# (C) 2018 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
    name: aws_resource_actions
    type: aggregate
    short_description: summarizes all "resource:actions" completed
    description:
      - Ansible callback plugin for collecting the AWS actions completed by all boto3 modules using
        AnsibleAWSModule in a playbook. Botocore endpoint logs need to be enabled for those modules, which can
        be done easily by setting debug_botocore_endpoint_logs to True for group/aws using module_defaults.
    requirements:
      - whitelisting in configuration - see examples section below for details.
"""

EXAMPLES = r"""
example: >
  To enable, add this to your ansible.cfg file in the defaults block
    [defaults]
    callback_whitelist = aws_resource_actions
sample output: >
  #
  # AWS ACTIONS: ['s3:PutBucketAcl', 's3:HeadObject', 's3:DeleteObject', 's3:PutObjectAcl', 's3:CreateMultipartUpload',
  #               's3:DeleteBucket', 's3:GetObject', 's3:DeleteObjects', 's3:CreateBucket', 's3:CompleteMultipartUpload',
  #               's3:ListObjectsV2', 's3:HeadBucket', 's3:UploadPart', 's3:PutObject']
"""

from ansible.module_utils.common.text.converters import to_native
from ansible.plugins.callback import CallbackBase


class CallbackModule(CallbackBase):
    """Callback plugin to collect and display AWS resource actions.

    This callback aggregates resource actions from task results. The resource_actions
    are captured by AnsibleAWSModule when debug_botocore_endpoint_logs is enabled.
    AnsibleAWSModule parses botocore.endpoint debug logs to extract API calls and
    adds them to the task result as resource_actions. This callback then collects
    those actions across all tasks and displays a summary.
    """

    CALLBACK_VERSION = 2.8
    CALLBACK_TYPE = "aggregate"
    CALLBACK_NAME = "amazon.aws.aws_resource_actions"
    CALLBACK_NEEDS_WHITELIST = True

    def __init__(self):
        self.aws_resource_actions = []
        super().__init__()

    def extend_aws_resource_actions(self, result):
        if result.get("resource_actions"):
            self.aws_resource_actions.extend(result["resource_actions"])

    def runner_on_ok(self, host, res):
        self.extend_aws_resource_actions(res)

    def runner_on_failed(self, host, res, ignore_errors=False):
        self.extend_aws_resource_actions(res)

    def v2_runner_item_on_ok(self, result):
        self.extend_aws_resource_actions(result._result)

    def v2_runner_item_on_failed(self, result):
        self.extend_aws_resource_actions(result._result)

    def playbook_on_stats(self, stats):
        if self.aws_resource_actions:
            self.aws_resource_actions = sorted(to_native(action) for action in set(self.aws_resource_actions))
            self._display.display(f"AWS ACTIONS: {self.aws_resource_actions}")



# Copyright (c) 2025, Timur Gadiev <tgadiev@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: to_prettytable
short_description: Format a list of dictionaries as an ASCII table
version_added: "10.7.0"
author: Timur Gadiev (@tgadiev)
description:
  - This filter takes a list of dictionaries and formats it as an ASCII table using the I(prettytable) Python library.
requirements:
  - prettytable
options:
  _input:
    description: A list of dictionaries to format.
    type: list
    elements: dictionary
    required: true
  column_order:
    description: List of column names to specify the order of columns in the table.
    type: list
    elements: string
  header_names:
    description: List of custom header names to use instead of dictionary keys.
    type: list
    elements: string
  column_alignments:
    description:
      - Dictionary where keys are column names and values are alignment settings. Valid alignment values are C(left), C(center),
        C(right), C(l), C(c), or C(r).
      - "For example, V({'name': 'left', 'id': 'right'}) aligns the C(name) column to the left and the C(id) column to the
        right."
    type: dictionary
"""

EXAMPLES = r"""
- name: Set a list of users
  ansible.builtin.set_fact:
    users:
      - name: Alice
        age: 25
        role: admin
      - name: Bob
        age: 30
        role: user

- name: Display a list of users as a table
  ansible.builtin.debug:
    msg: >-
      {{
        users | community.general.to_prettytable
      }}

- name: Display a table with custom column ordering
  ansible.builtin.debug:
    msg: >-
      {{
        users | community.general.to_prettytable(
          column_order=['role', 'name', 'age']
        )
      }}

- name: Display a table with selective column output (only show name and role fields)
  ansible.builtin.debug:
    msg: >-
      {{
        users | community.general.to_prettytable(
          column_order=['name', 'role']
        )
      }}

- name: Display a table with custom headers
  ansible.builtin.debug:
    msg: >-
      {{
        users | community.general.to_prettytable(
          header_names=['User Name', 'User Age', 'User Role']
        )
      }}

- name: Display a table with custom alignments
  ansible.builtin.debug:
    msg: >-
      {{
        users | community.general.to_prettytable(
          column_alignments={'name': 'center', 'age': 'right', 'role': 'left'}
        )
      }}

- name: Combine multiple options
  ansible.builtin.debug:
    msg: >-
      {{
        users | community.general.to_prettytable(
          column_order=['role', 'name', 'age'],
          header_names=['Position', 'Full Name', 'Years'],
          column_alignments={'name': 'center', 'age': 'right', 'role': 'left'}
        )
      }}
"""

RETURN = r"""
_value:
  description: The formatted ASCII table.
  type: string
"""

try:
    import prettytable

    HAS_PRETTYTABLE = True
except ImportError:
    HAS_PRETTYTABLE = False

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.text.converters import to_text


class TypeValidationError(AnsibleFilterError):
    """Custom exception for type validation errors.

    Args:
        obj: The object with incorrect type
        expected: Description of expected type
    """

    def __init__(self, obj, expected):
        type_name = "string" if isinstance(obj, str) else type(obj).__name__
        super().__init__(f"Expected {expected}, got a {type_name}")


def _validate_list_param(param, param_name, ensure_strings=True):
    """Validate a parameter is a list and optionally ensure all elements are strings.

    Args:
        param: The parameter to validate
        param_name: The name of the parameter for error messages
        ensure_strings: Whether to check that all elements are strings

    Raises:
        AnsibleFilterError: If validation fails
    """
    # Map parameter names to their original error message format
    error_messages = {"column_order": "a list of column names", "header_names": "a list of header names"}

    # Use the specific error message if available, otherwise use a generic one
    error_msg = error_messages.get(param_name, f"a list for {param_name}")

    if not isinstance(param, list):
        raise TypeValidationError(param, error_msg)

    if ensure_strings:
        for item in param:
            if not isinstance(item, str):
                # Maintain original error message format
                if param_name == "column_order":
                    error_msg = "a string for column name"
                elif param_name == "header_names":
                    error_msg = "a string for header name"
                else:
                    error_msg = f"a string for {param_name} element"
                raise TypeValidationError(item, error_msg)


def _match_key(item_dict, lookup_key):
    """Find a matching key in a dictionary, handling type conversion.

    Args:
        item_dict: Dictionary to search in
        lookup_key: Key to look for, possibly needing type conversion

    Returns:
        The matching key or None if no match found
    """
    # Direct key match
    if lookup_key in item_dict:
        return lookup_key

    # Try boolean conversion for 'true'/'false' strings
    if isinstance(lookup_key, str):
        if lookup_key.lower() == "true" and True in item_dict:
            return True
        if lookup_key.lower() == "false" and False in item_dict:
            return False

        # Try numeric conversion for string numbers
        if lookup_key.isdigit() and int(lookup_key) in item_dict:
            return int(lookup_key)

    # No match found
    return None


def _build_key_maps(data):
    """Build mappings between string keys and original keys.

    Args:
        data: List of dictionaries with keys to map

    Returns:
        Tuple of (key_map, reverse_key_map)
    """
    key_map = {}
    reverse_key_map = {}

    # Check if the data list is not empty
    if not data:
        return key_map, reverse_key_map

    first_dict = data[0]
    for orig_key in first_dict.keys():
        # Store string version of the key
        str_key = to_text(orig_key)
        key_map[str_key] = orig_key
        # Also store lowercase version for case-insensitive lookups
        reverse_key_map[str_key.lower()] = orig_key

    return key_map, reverse_key_map


def _configure_alignments(table, field_names, column_alignments):
    """Configure column alignments for the table.

    Args:
        table: The PrettyTable instance to configure
        field_names: List of field names to align
        column_alignments: Dict of column alignments
    """

    if not isinstance(column_alignments, dict):
        return

    for col_name, alignment in column_alignments.items():
        if col_name in field_names:
            # We already validated alignment is a string and a valid value in the main function
            # Just apply it here
            alignment = alignment.lower()
            table.align[col_name] = alignment[0]


def to_prettytable(data, *args, **kwargs):
    """Convert a list of dictionaries to an ASCII table.

    Args:
        data: List of dictionaries to format
        *args: Optional list of column names to specify column order
        **kwargs: Optional keyword arguments:
            - column_order: List of column names to specify the order
            - header_names: List of custom header names
            - column_alignments: Dict of column alignments (left, center, right)

    Returns:
        String containing the ASCII table
    """
    if not HAS_PRETTYTABLE:
        raise AnsibleFilterError('You need to install "prettytable" Python module to use this filter')

    # === Input validation ===
    # Validate list type
    if not isinstance(data, list):
        raise TypeValidationError(data, "a list of dictionaries")

    # Validate dictionary items if list is not empty
    if data and not all(isinstance(item, dict) for item in data):
        invalid_item = next((item for item in data if not isinstance(item, dict)), None)
        raise TypeValidationError(invalid_item, "all items in the list to be dictionaries")

    # Get sample dictionary to determine fields - empty if no data
    sample_dict = data[0] if data else {}
    max_fields = len(sample_dict)

    # === Process column order ===
    # Handle both positional and keyword column_order
    column_order = kwargs.pop("column_order", None)

    # Check for conflict between args and column_order
    if args and column_order is not None:
        raise AnsibleFilterError("Cannot use both positional arguments and the 'column_order' keyword argument")

    # Use positional args if provided
    if args:
        column_order = list(args)

    # Validate column_order
    if column_order is not None:
        _validate_list_param(column_order, "column_order")

        # Validate column_order doesn't exceed the number of fields (skip if data is empty)
        if data and len(column_order) > max_fields:
            raise AnsibleFilterError(
                f"'column_order' has more elements ({len(column_order)}) than available fields in data ({max_fields})"
            )

    # === Process headers ===
    # Determine field names and ensure they are strings
    if column_order:
        field_names = column_order
    else:
        # Use field names from first dictionary, ensuring all are strings
        field_names = [to_text(k) for k in sample_dict]

    # Process custom headers
    header_names = kwargs.pop("header_names", None)
    if header_names is not None:
        _validate_list_param(header_names, "header_names")

        # Validate header_names doesn't exceed the number of fields (skip if data is empty)
        if data and len(header_names) > max_fields:
            raise AnsibleFilterError(
                f"'header_names' has more elements ({len(header_names)}) than available fields in data ({max_fields})"
            )

        # Validate that column_order and header_names have the same size if both provided
        if column_order is not None and len(column_order) != len(header_names):
            raise AnsibleFilterError(
                f"'column_order' and 'header_names' must have the same number of elements. "
                f"Got {len(column_order)} columns and {len(header_names)} headers."
            )

    # === Process alignments ===
    # Get column alignments and validate
    column_alignments = kwargs.pop("column_alignments", {})
    valid_alignments = {"left", "center", "right", "l", "c", "r"}

    # Validate column_alignments is a dictionary
    if not isinstance(column_alignments, dict):
        raise TypeValidationError(column_alignments, "a dictionary for column_alignments")

    # Validate column_alignments keys and values
    for key, value in column_alignments.items():
        # Check that keys are strings
        if not isinstance(key, str):
            raise TypeValidationError(key, "a string for column_alignments key")

        # Check that values are strings
        if not isinstance(value, str):
            raise TypeValidationError(value, "a string for column_alignments value")

        # Check that values are valid alignments
        if value.lower() not in valid_alignments:
            raise AnsibleFilterError(
                f"Invalid alignment '{value}' in 'column_alignments'. "
                f"Valid alignments are: {', '.join(sorted(valid_alignments))}"
            )

    # Validate column_alignments doesn't have more keys than fields (skip if data is empty)
    if data and len(column_alignments) > max_fields:
        raise AnsibleFilterError(
            f"'column_alignments' has more elements ({len(column_alignments)}) than available fields in data ({max_fields})"
        )

    # Check for unknown parameters
    if kwargs:
        raise AnsibleFilterError(f"Unknown parameter(s) for to_prettytable filter: {', '.join(sorted(kwargs))}")

    # === Build the table ===
    table = prettytable.PrettyTable()

    # Set the field names for display
    display_names = header_names if header_names is not None else field_names
    table.field_names = [to_text(name) for name in display_names]

    # Configure alignments after setting field_names
    _configure_alignments(table, display_names, column_alignments)

    # Build key maps only if not using explicit column_order and we have data
    key_map = {}
    reverse_key_map = {}
    if not column_order and data:  # Only needed when using original dictionary keys and we have data
        key_map, reverse_key_map = _build_key_maps(data)

    # If we have an empty list with no custom parameters, return a simple empty table
    if not data and not column_order and not header_names and not column_alignments:
        return "++\n++"

    # Process each row if we have data
    for item in data:
        row = []
        for col in field_names:
            # Try direct mapping first
            if col in key_map:
                row.append(item.get(key_map[col], ""))
            else:
                # Try to find a matching key in the item
                matched_key = _match_key(item, col)
                if matched_key is not None:
                    row.append(item.get(matched_key, ""))
                else:
                    # Try case-insensitive lookup as last resort
                    lower_col = col.lower() if isinstance(col, str) else str(col).lower()
                    if lower_col in reverse_key_map:
                        row.append(item.get(reverse_key_map[lower_col], ""))
                    else:
                        # No match found
                        row.append("")
        table.add_row(row)

    return to_text(table)


class FilterModule:
    """Ansible core jinja2 filters."""

    def filters(self):
        return {"to_prettytable": to_prettytable}



# Copyright (c) 2017-2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.dns.plugins.module_utils._record import RecordIDT
from ansible_collections.community.dns.plugins.module_utils._record_set import (
    RecordSetIDT,
)

if t.TYPE_CHECKING:  # pragma: no cover
    from collections.abc import Mapping

    from ._record import DNSRecord
    from ._record_set import DNSRecordSet


ZoneIDT = t.TypeVar("ZoneIDT")
ZoneIDT_co = t.TypeVar("ZoneIDT_co", covariant=True)


class IDNSZone(t.Protocol, t.Generic[ZoneIDT_co]):  # pragma: no cover
    @property
    def id(self) -> ZoneIDT_co: ...

    @property
    def name(self) -> str: ...

    @property
    def info(self) -> Mapping[str, t.Any]: ...


class DNSZone(t.Generic[ZoneIDT]):
    def __init__(
        self,
        *,
        zone_id: ZoneIDT,
        name: str,
        info: dict[str, t.Any] | None = None,
    ) -> None:
        self.id = zone_id
        self.name = name
        self.info: dict[str, t.Any] = info or {}

    def __str__(self) -> str:
        data = []
        if self.id is not None:
            data.append(f"id: {self.id}")
        data.append(f"name: {self.name}")
        data.append(f"info: {self.info}")
        return "DNSZone(" + ", ".join(data) + ")"

    def __repr__(self) -> str:
        return self.__str__()


class DNSZoneWithRecords(t.Generic[ZoneIDT, RecordIDT]):
    def __init__(
        self,
        zone: DNSZone[ZoneIDT],
        records: list[DNSRecord[RecordIDT]],
    ) -> None:
        self.zone = zone
        self.records = records

    def __str__(self) -> str:
        return f"({self.zone}, {self.records})"

    def __repr__(self) -> str:
        return f"DNSZoneWithRecords({self.zone!r}, {self.records!r})"


class DNSZoneWithRecordSets(t.Generic[ZoneIDT, RecordSetIDT, RecordIDT]):
    def __init__(
        self,
        zone: DNSZone[ZoneIDT],
        record_sets: list[DNSRecordSet[RecordSetIDT, RecordIDT]],
    ) -> None:
        self.zone = zone
        self.record_sets = record_sets

    def __str__(self) -> str:
        return f"({self.zone}, {self.record_sets})"

    def __repr__(self) -> str:
        return f"DNSZoneWithRecordSets({self.zone!r}, {self.record_sets!r})"



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Akini Ross (@akinross) <akinross@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_pod_profile
version_added: "2.11.0"
short_description: Manage Pod Profiles on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Pod Profiles on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.2 (NDO v4.4) and later.
author:
- Akini Ross (@akinross)
options:
  template:
    description:
    - The name of the template.
    - The template must be a Fabric Resource Policy template.
    - This parameter or O(template_id) is required.
    type: str
  template_id:
    description:
    - The ID of the template.
    - The template must be a Fabric Resource Policy template.
    - This parameter or O(template) is required.
    type: str
  name:
    description:
    - The name of the Pod Profile.
    type: str
    aliases: [ pod_profile ]
  uuid:
    description:
    - The UUID of the Pod Profile.
    - This parameter is required when the O(name) needs to be updated.
    type: str
    aliases: [ pod_profile_uuid ]
  description:
    description:
    - The description of the Pod Profile.
    type: str
  pods:
    description:
    - A list of pod IDs to be used in the Pod Profile.
    - When provided, the Pod Profile will apply to pods specified in the range.
    - When an empty list O(pods=[]) is provided, the Pod Profile will apply to all pods.
    - When not provided on initial create, the Pod Profile will apply to all pods.
    type: list
    elements: int
    aliases: [ blocks ]
  pod_settings_uuid:
    description:
    - The UUID of the Pod Settings Policy to be used.
    type: str
  pod_settings:
    description:
    - The reference of the Pod Settings Policy to be used.
    type: dict
    suboptions:
      name:
        description:
        - The name of the Pod Settings Policy.
        type: str
        required: true
      template:
        description:
        - The name of the Pod Settings template.
        - This parameter or O(pod_settings.template_id) is required when O(pod_settings) is used.
        type: str
      template_id:
        description:
        - The ID of the Pod Settings template.
        - This parameter or O(pod_settings.template) is required when O(pod_settings) is used.
        type: str
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Fabric Resource Policy template.
- The O(pod_settings) or O(pod_settings_uuid) must exist before using it with this module in your playbook.
  Use M(cisco.mso.ndo_pod_settings) to create the Pod Settings Policy.
seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.ndo_pod_settings
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a Pod Profile
  cisco.mso.ndo_pod_profile:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_pod_profile
    pod_settings:
      name: ansible_pod_settings
      template: ansible_fabric_policy_template
    state: present
  register: create_pod_profile

- name: Update a Pod Profile using name
  cisco.mso.ndo_pod_profile:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_pod_profile
    pod_settings:
      name: ansible_pod_settings
      template: ansible_fabric_policy_template
    description: Updated Pod Profile
    pods:
      - 1
      - 2
    state: present

- name: Update a Pod Profile using UUID
  cisco.mso.ndo_pod_profile:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_pod_profile_changed
    uuid: "{{ create_pod_profile.current.uuid }}"
    pod_settings_uuid: "{{ create_pod_settings.current.uuid }}"
    state: present

- name: Query a Pod Profile using name
  cisco.mso.ndo_pod_profile:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_pod_profile
    state: query
  register: query_with_name

- name: Query a Pod Profile using UUID
  cisco.mso.ndo_pod_profile:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    uuid: "{{ create_pod_profile.current.uuid }}"
    state: query
  register: query_with_uuid

- name: Query all Pod Profile in a template
  cisco.mso.ndo_pod_profile:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    state: query
  register: query_all_objects

- name: Delete a Pod Profile using Name
  cisco.mso.ndo_pod_profile:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_pod_profile
    state: absent

- name: Delete a Pod Profile using UUID
  cisco.mso.ndo_pod_profile:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template_id: ansible_test
    uuid: "{{ create_pod_profile.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.templates import MSOTemplates
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data
import copy

POD_SETTINGS_CACHE = {}


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str"),
        template_id=dict(type="str"),
        name=dict(type="str", aliases=["pod_profile"]),
        uuid=dict(type="str", aliases=["pod_profile_uuid"]),
        description=dict(type="str"),
        pods=dict(type="list", elements="int", aliases=["blocks"]),  # UI uses blocks but chose pods since it is more descriptive
        pod_settings=dict(
            type="dict",
            options=dict(
                name=dict(type="str", required=True),
                template=dict(type="str"),
                template_id=dict(type="str"),
            ),
            required_one_of=[
                ["template", "template_id"],
            ],
            mutually_exclusive=[
                ("template", "template_id"),
            ],
        ),
        pod_settings_uuid=dict(type="str"),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ("template", "template_id"),
            ("pod_settings", "pod_settings_uuid"),
        ],
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
            ["state", "present", ["pod_settings", "pod_settings_uuid"], True],
        ],
        required_one_of=[
            ["template", "template_id"],
        ],
    )

    mso = MSOModule(module)
    mso_templates = MSOTemplates(mso)

    template_name = module.params.get("template")
    template_id = module.params.get("template_id")
    name = module.params.get("name")
    uuid = module.params.get("uuid")
    description = module.params.get("description")
    pods = module.params.get("pods")
    pod_settings = module.params.get("pod_settings")
    pod_settings_uuid = module.params.get("pod_settings_uuid")
    state = module.params.get("state")

    mso_template = mso_templates.get_template("fabric_resource", template_name, template_id)
    mso_template.validate_template("fabricResource")

    match = mso_template.get_pod_profile_object(uuid, name)

    if (uuid or name) and match:
        set_pod_profile_policy_names(mso, mso_templates, mso_template, match.details)
        mso.existing = mso.previous = copy.deepcopy(match.details)  # Query a specific object
    elif match:
        mso.existing = [set_pod_profile_policy_names(mso, mso_templates, mso_template, obj) for obj in match]  # Query all objects

    pod_profile_path = "/fabricResourceTemplate/template/podProfiles/{0}".format(match.index if match else "-")

    ops = []

    if state == "present":

        mso_values = {
            "name": name,
            "description": description,
        }

        if pods:
            mso_values["blocks"] = pods
            mso_values["kind"] = "podRange"
        elif pods == []:
            mso_values["kind"] = "all"

        if pod_settings_uuid:
            mso_values["policy"] = pod_settings_uuid
        else:
            pod_settings_mso_template = mso_templates.get_template("fabric_policy", pod_settings.get("template"), pod_settings.get("template_id"))
            mso_values["policy"] = pod_settings_mso_template.get_pod_settings_object(
                pod_settings_uuid, pod_settings.get("name"), fail_module=True
            ).details.get("uuid")

        if match:
            remove_data = []
            if pods == []:
                remove_data.append("blocks")

            append_update_ops_data(ops, match.details, pod_profile_path, mso_values, remove_data)
            mso.sanitize(mso_values, collate=True, unwanted=remove_data)
            set_pod_profile_policy_names(mso, mso_templates, mso_template, mso.proposed)
        else:
            if not pods:
                mso_values["kind"] = "all"
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=pod_profile_path, value=mso.sent))

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path=pod_profile_path))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        match = mso_template.get_pod_profile_object(uuid, name, search_object=response)
        if match:
            set_pod_profile_policy_names(mso, mso_templates, mso_template, match.details)
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        if mso.proposed:
            set_pod_profile_policy_names(mso, mso_templates, mso_template, mso.proposed)
        mso.existing = mso.proposed if state == "present" else {}
    mso.exit_json()


def set_pod_profile_policy_names(mso, mso_templates, mso_template, pod_profile):

    # TODO: leverage update_config_with_template_and_references when podPolicyGroup type is supported by objects API endpoint
    # "mso/api/v1/templates/objects?type=podPolicyGroup&uuid=0809339e-65f7-4000-a23a-a39241865db8"
    # {
    #   "code": 400,
    #   "message": "unsupported policy type: podPolicyGroup"
    # }
    # reference_dict = {
    #     "policy": {
    #         "name": "pod_settings",
    #         "reference": "policy",
    #         "type": "podPolicyGroup",
    #     }
    # }
    # if pod_profile and pod_profile.get("policy"):
    #     mso_template.update_config_with_template_and_references(pod_profile, reference_dict)

    # Workaround template looping until podPolicyGroup type is supported by objects API endpoint
    # Check if a policy is set in the pod profile, this is the UUID of the pod settings policy

    pod_profile["templateName"] = mso_template.template_name
    pod_profile["templateId"] = mso_template.template_id
    if pod_profile.get("policy"):
        pod_settings_uuid = pod_profile.get("policy")

        # Check if the pod settings UUID is already in the cache
        if pod_settings_uuid in POD_SETTINGS_CACHE:
            pod_profile_details = POD_SETTINGS_CACHE[pod_settings_uuid]
            pod_profile["policyName"] = pod_profile_details.get("policyName")
            pod_profile["policyTemplateName"] = pod_profile_details.get("policyTemplateName")
            pod_profile["policyTemplateId"] = pod_profile_details.get("policyTemplateId")
            return pod_profile

        # Retrieve a summary of all fabric_policy templates from NDO and loop through all templates to find the pod settings policy
        for template in MSOTemplate(mso, "fabric_policy").template:
            for policy in template.get("policies", []):
                # Only retrieve the template details if the fabric_policy template has a pod settings policy defined
                if policy.get("objType") == "podPolicyGroup" and policy.get("count") > 0:
                    pod_settings_mso_template = mso_templates.get_template("fabric_policy", template.get("templateName"), template.get("templateId"))
                    match = pod_settings_mso_template.get_pod_settings_object(pod_settings_uuid)
                    if match:
                        pod_profile["policyName"] = match.details.get("name")
                        pod_profile["policyTemplateName"] = pod_settings_mso_template.template_name
                        pod_profile["policyTemplateId"] = pod_settings_mso_template.template_id
                        POD_SETTINGS_CACHE[pod_settings_uuid] = {
                            "policyName": match.details.get("name"),
                            "policyTemplateName": pod_settings_mso_template.template_name,
                            "policyTemplateId": pod_settings_mso_template.template_id,
                        }
                        return pod_profile


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Hannes Ljungberg <hannes.ljungberg@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_swarm_service_info

short_description: Retrieves information about docker services from a Swarm Manager

description:
  - Retrieves information about a docker service.
  - Essentially returns the output of C(docker service inspect <name>).
  - Must be executed on a host running as Swarm Manager, otherwise the module will fail.
extends_documentation_fragment:
  - community.docker._docker
  - community.docker._docker.docker_py_2_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker
  - community.docker._attributes.info_module
  - community.docker._attributes.idempotent_not_modify_state

options:
  name:
    description:
      - The name of the service to inspect.
    type: str
    required: true

author:
  - Hannes Ljungberg (@hannseman)

requirements:
  - "L(Docker SDK for Python,https://docker-py.readthedocs.io/en/stable/) >= 2.0.0"
  - "Docker API >= 1.25"
"""

EXAMPLES = r"""
---
- name: Get info from a service
  community.docker.docker_swarm_service_info:
    name: myservice
  register: result
"""

RETURN = r"""
exists:
  description:
    - Returns whether the service exists.
  type: bool
  returned: always
  sample: true
service:
  description:
    - A dictionary representing the current state of the service. Matches the C(docker service inspect) output.
    - Will be V(none) if service does not exist.
  returned: always
  type: dict
"""

import traceback
import typing as t

try:
    from docker.errors import DockerException
except ImportError:
    # missing Docker SDK for Python handled in ansible.module_utils.docker.common
    pass

from ansible_collections.community.docker.plugins.module_utils._common import (
    RequestException,
)
from ansible_collections.community.docker.plugins.module_utils._swarm import (
    AnsibleDockerSwarmClient,
)


def get_service_info(client: AnsibleDockerSwarmClient) -> dict[str, t.Any] | None:
    service = client.module.params["name"]
    return client.get_service_inspect(service_id=service, skip_missing=True)


def main() -> None:
    argument_spec = {
        "name": {"type": "str", "required": True},
    }

    client = AnsibleDockerSwarmClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
        min_docker_version="2.0.0",
    )

    client.fail_task_if_not_swarm_manager()

    try:
        service = get_service_info(client)

        client.module.exit_json(changed=False, service=service, exists=bool(service))
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )
    except RequestException as e:
        client.fail(
            f"An unexpected requests error occurred when Docker SDK for Python tried to talk to the docker daemon: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_batch_endpoint
version_added: "3.18.0"
short_description: Create, Delete or regenerate keys for an Azure Machine Learning Batch Endpoint
description:
    - Create, Delete or regenerate keys for an Azure Machine Learning Batch Endpoint.
options:
    name:
        description:
            - Name of the Azure ML batch_endpoint.
        type: str
        required: true
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    resource_definition:
        description:
            - Batch Endpoint definition in YAML
        type: str
        required: false
    description:
        description:
            - Description of the batch_endpoint.
        type: str
    state:
        description:
            - State of the Batch Endpoint. Use C(present) to create
              or update.
        default: present
        type: str
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Create ML Batch Endpoint
  azure.azcollection.azure_rm_ml_batch_endpoint:
    name: endpoint-xxxxxxxxxx-ansible
    resource_group: xxxxx-ml-batch_endpoint
    ml_workspace: workspace-xxxxxxxxxx
    resource_definition: |
      $schema: https://azuremlschemas.azureedge.net/latest/batchEndpoint.schema.json
      name: hello-batch
      description: A hello world endpoint for component deployments.
      auth_mode: aad_token
'''

RETURN = '''
changed:
    description:
        - Whether the resource is changed.
    returned: always
    type: bool
    sample: false
ml_batch_endpoint:
    description:
        - Batch Endpoint that was just created or updated.
    returned: always
    type: dict
    sample: {
        "auth_mode": "aad_token",
        "description": "A hello world endpoint for component deployments.",
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-batch_endpoint_invoke/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/batchEndpoints/endpoint-xxxxxxxxxx-ansible",
        "location": "eastus",
        "name": "endpoint-xxxxxxxxxx-ansible",
        "properties": {
            "BatchEndpointCreationApiVersion": "2023-10-01",
            "azureml.onlineendpointid": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-batch_endpoint_invoke/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/batchEndpoints/endpoint-xxxxxxxxxx-ansible"
        },
        "provisioning_state": "Succeeded",
        "scoring_uri": "https://endpoint-xxxxxxxxxx-ansible.eastus.inference.ml.azure.com/jobs",
        "tags": {}
    }
'''  # NOQA


try:
    import io
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
    from azure.ai.ml.entities._load_functions import load_batch_endpoint
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLBatchEndpoint(MLClientCommon):

    UPDATE_OVERRIDES = ['description',
                        'tags']

    PARAM_OVERRIDES = UPDATE_OVERRIDES + ['name']

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=True,
            ),
            resource_definition=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            description=dict(
                type='str',
                required=False,
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent'],
            ),
        )

        self._client = None
        self.ml_registry = None
        self.state = None

        self.results = dict(
            changed=False,
            compare=[],
            ml_batch_endpoint={}
        )

        required_if = [
            ('state', 'present', ['resource_definition'])
        ]

        super(AzureRMMLBatchEndpoint, self).__init__(self.module_arg_spec,
                                                     supports_check_mode=True,
                                                     required_if=required_if,
                                                     )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        changed = False

        ml_batch_endpoint_info = self.get(name=self.name)

        if self.state == 'present':
            if self.resource_definition:
                resource_definition = io.StringIO(self.resource_definition)

            if ml_batch_endpoint_info:
                # Update
                params_override = self.update_params(kwargs,
                                                     self.UPDATE_OVERRIDES,
                                                     ml_batch_endpoint_info)

                # Generate ml_batch_endpoint based on ansible args passed in.
                ml_batch_endpoint = load_batch_endpoint(resource_definition,
                                                        params_override=params_override)
                ml_batch_endpoint_info = self.entity_to_dict(ml_batch_endpoint_info)
                changed = not self.default_compare({},
                                                   self.entity_to_dict(ml_batch_endpoint),
                                                   ml_batch_endpoint_info,
                                                   '',
                                                   self.results)
                if changed and not self.check_mode:
                    response = self.client.begin_create_or_update(ml_batch_endpoint)
                    ml_batch_endpoint_object = self.get_poller_result(response)
                    ml_batch_endpoint_info = self.entity_to_dict(ml_batch_endpoint_object)
            else:
                # Create
                params_override = self.update_params(kwargs, self.PARAM_OVERRIDES)

                # Generate ml_batch_endpoint based on ansible args passed in.
                ml_batch_endpoint = load_batch_endpoint(source=resource_definition,
                                                        params_override=params_override)

                changed = True
                if not self.check_mode:
                    response = self.client.begin_create_or_update(ml_batch_endpoint)
                    ml_batch_endpoint_object = self.get_poller_result(response)
                    ml_batch_endpoint_info = self.entity_to_dict(ml_batch_endpoint_object)
        elif self.state == 'absent':
            if ml_batch_endpoint_info:
                # Delete
                changed = True
                if not self.check_mode:
                    response = self.client.batch_endpoints.begin_delete(name=self.name)
                    ml_batch_endpoint_info = self.get_poller_result(response)

        self.results['ml_batch_endpoint'] = ml_batch_endpoint_info
        self.results['changed'] = changed
        return self.results

    def update_params(self, kwargs, param_overrides, ml_batch_endpoint=None):
        params_override = []

        # If ml_batch_endpoint is defined use those values as a base.
        # Update can't change some of these values but we need
        # to keep the same values as before the update.
        if ml_batch_endpoint:
            items = self.entity_to_dict(ml_batch_endpoint).items()
            params_override = [{k: v} for k, v in items]

        # Update values based on what param_overrides says we can update.
        for key in list(self.module_arg_spec.keys()):
            if kwargs[key] is not None and key in param_overrides:
                params_override.append({key: kwargs[key]})
        return params_override

    def get(self, name, as_dict=False):
        try:
            # Attempt to retrieve ml_batch_endpoint based on name
            ml_batch_endpoint_info = self.client.batch_endpoints.get(name=name)
            if as_dict:
                ml_batch_endpoint_info = self.entity_to_dict(ml_batch_endpoint_info)
        except ResourceNotFoundError:
            ml_batch_endpoint_info = None
        return ml_batch_endpoint_info


def main():
    AzureRMMLBatchEndpoint()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: GPL-3.0-only
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_dns_zone
short_description: Manage Windows Server DNS Zones
author: Joe Zollo (@joezollo)
version_added: 2.6.0
requirements:
  - This module requires Windows Server 2012R2 or Newer
description:
  - Manage Windows Server DNS Zones
  - Adds, Removes and Modifies DNS Zones - Primary, Secondary, Forwarder & Stub
  - Task should be delegated to a Windows DNS Server
options:
  name:
    description:
      - Fully qualified name of the DNS zone.
    type: str
    required: true
  type:
    description:
      - Specifies the type of DNS zone.
      - When l(type=secondary), the DNS server will immediately attempt to
        perform a zone transfer from the servers in this list. If this initial
        transfer fails, then the zone will be left in an unworkable state.
        This module does not verify the initial transfer.
    type: str
    choices: [ primary, secondary, stub, forwarder ]
  dynamic_update:
    description:
      - Specifies how a zone handles dynamic updates.
      - Secure DNS updates are available only for Active Directory-integrated
        zones.
      - When not specified during new zone creation, Windows will default this
        to l(none).
    type: str
    choices: [ secure, none, nonsecureandsecure ]
  state:
    description:
      - Specifies the desired state of the DNS zone.
      - When l(state=present) the module will attempt to create the specified
        DNS zone if it does not already exist.
      - When l(state=absent), the module will remove the specified DNS
        zone and all subsequent DNS records.
    type: str
    default: present
    choices: [ present, absent ]
  forwarder_timeout:
    description:
      - Specifies a length of time, in seconds, that a DNS server waits for a
        remote DNS server to resolve a query.
      - Accepts integer values between 0 and 15.
      - If the provided value is not valid, it will be omitted and a warning
        will be issued.
    type: int
  replication:
    description:
      - Specifies the replication scope for the DNS zone.
      - l(replication=forest) will replicate the DNS zone to all domain
        controllers in the Active Directory forest.
      - l(replication=domain) will replicate the DNS zone to all domain
        controllers in the Active Directory domain.
      - l(replication=legacy) will replicate the DNS zone to all domain
        controllers in the Active Directory domain and to legacy clients.
      - l(replication=none) disables Active Directory integration and
        creates a local file with the name of the zone.
      - This is the equivalent of selecting l(store the zone in Active
        Directory) in the GUI.
    type: str
    choices: [ forest, domain, legacy, none ]
  directory_partition:
    description:
      - Specifies the name of an existing custom Active Directory
        application directory partition where the zone should be stored.
      - When specified, the zone will use a custom replication scope
        instead of the standard domain or forest partitions.
      - This is mutually exclusive with O(replication) values of
        C(forest), C(domain), and C(legacy).
      - Requires the target directory partition to already exist.
    type: str
    version_added: 3.7.0
  dns_servers:
    description:
      - Specifies an list of IP addresses of the primary servers of the zone.
      - DNS queries for a forwarded zone are sent to primary servers.
      - Required if l(type=secondary), l(type=forwarder) or l(type=stub),
        otherwise ignored.
      - At least one server is required.
    elements: str
    type: list
'''

EXAMPLES = r'''
- name: Ensure primary zone is present
  ansible.windows.win_dns_zone:
    name: wpinner.euc.vmware.com
    replication: domain
    type: primary
    state: present

- name: Ensure DNS zone is absent
  ansible.windows.win_dns_zone:
    name: jamals.euc.vmware.com
    state: absent

- name: Ensure forwarder has specific DNS servers
  ansible.windows.win_dns_zone:
    name: jamals.euc.vmware.com
    type: forwarder
    dns_servers:
      - 10.245.51.100
      - 10.245.51.101
      - 10.245.51.102

- name: Ensure stub zone has specific DNS servers
  ansible.windows.win_dns_zone:
    name: virajp.euc.vmware.com
    type: stub
    dns_servers:
      - 10.58.2.100
      - 10.58.2.101

- name: Ensure stub zone is converted to a secondary zone
  ansible.windows.win_dns_zone:
    name: virajp.euc.vmware.com
    type: secondary

- name: Ensure secondary zone is present with no replication
  ansible.windows.win_dns_zone:
    name: dgemzer.euc.vmware.com
    type: secondary
    replication: none
    dns_servers:
      - 10.19.20.1

- name: Ensure secondary zone is converted to a primary zone
  ansible.windows.win_dns_zone:
    name: dgemzer.euc.vmware.com
    type: primary
    replication: none
    dns_servers:
      - 10.19.20.1

- name: Ensure primary DNS zone is present without replication
  ansible.windows.win_dns_zone:
    name: basavaraju.euc.vmware.com
    replication: none
    type: primary

- name: Ensure primary DNS zone has nonsecureandsecure dynamic updates enabled
  ansible.windows.win_dns_zone:
    name: basavaraju.euc.vmware.com
    replication: none
    dynamic_update: nonsecureandsecure
    type: primary

- name: Ensure DNS zone is absent
  ansible.windows.win_dns_zone:
    name: marshallb.euc.vmware.com
    state: absent

- name: Ensure primary zone is stored in a custom directory partition
  ansible.windows.win_dns_zone:
    name: custom.euc.vmware.com
    type: primary
    directory_partition: CustomDNSPartition

- name: Ensure forwarder zone is stored in a custom directory partition
  ansible.windows.win_dns_zone:
    name: forwarder.euc.vmware.com
    type: forwarder
    directory_partition: CustomDNSPartition
    dns_servers:
      - 10.245.51.100

- name: Ensure DNS zones are absent
  ansible.windows.win_dns_zone:
    name: "{{ item }}"
    state: absent
  loop:
    - jamals.euc.vmware.com
    - dgemzer.euc.vmware.com
    - wpinner.euc.vmware.com
    - marshallb.euc.vmware.com
    - basavaraju.euc.vmware.com
'''

RETURN = r'''
zone:
  description: New/Updated DNS zone parameters
  returned: When l(state=present)
  type: dict
  sample:
    name:
    type:
    dynamic_update:
    reverse_lookup:
    forwarder_timeout:
    paused:
    shutdown:
    zone_file:
    replication:
    directory_partition:
    dns_servers:
'''



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c), Jonathan Mainguy <jon@soh.re>, 2015
# Most of this was originally added by Sven Schliesing @muffl0n in the mysql_user.py module
#
# Simplified BSD License (see simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)

from __future__ import (absolute_import, division, print_function)
from functools import reduce
__metaclass__ = type

import os
import configparser

from ansible.module_utils.common.text.converters import to_native

try:
    import pymysql as mysql_driver
    _mysql_cursor_param = 'cursor'
except ImportError:
    try:
        # mysqlclient is called MySQLdb
        import MySQLdb as mysql_driver
        import MySQLdb.cursors
        _mysql_cursor_param = 'cursorclass'
    except ImportError:
        mysql_driver = None

mysql_driver_fail_msg = ('A MySQL module is required: for Python 2.7 either PyMySQL, or '
                         'MySQL-python, or for Python 3.X mysqlclient or PyMySQL. '
                         'Consider setting ansible_python_interpreter to use '
                         'the intended Python version.')

from ansible_collections.ansible.mysql.plugins.module_utils.database import mysql_quote_identifier


def get_connector_name(connector):
    """ (class) -> str
    Return the name of the connector (pymysql or mysqlclient (MySQLdb))
    or 'Unknown' if not pymysql or MySQLdb. When adding a
    connector here, also modify get_connector_version.
    """
    if connector is None or not hasattr(connector, '__name__'):
        return 'Unknown'

    return connector.__name__


def get_connector_version(connector):
    """ (class) -> str
    Return the version of pymysql or mysqlclient (MySQLdb).
    Return 'Unknown' if the connector name is unknown.
    """

    if connector is None:
        return 'Unknown'

    connector_name = get_connector_name(connector)

    if connector_name == 'pymysql':
        # pymysql has two methods:
        # - __version__ that returns the string: 0.7.11.None
        # - VERSION that returns the tuple (0, 7, 11, None)
        v = connector.VERSION[:3]
        return '.'.join(map(str, v))
    elif connector_name == 'MySQLdb':
        # version_info returns the tuple (2, 1, 1, 'final', 0)
        v = connector.version_info[:3]
        return '.'.join(map(str, v))
    else:
        return 'Unknown'


def parse_from_mysql_config_file(cnf):
    # Default values of comment_prefix is '#' and ';'.
    # '!' added to prevent a parsing error
    # when a config file contains !includedir parameter.
    cp = configparser.ConfigParser(comment_prefixes=('#', ';', '!'))
    cp.read(cnf)
    return cp


def mysql_connect(module, login_user=None, login_password=None, config_file='', ssl_cert=None,
                  ssl_key=None, ssl_ca=None, db=None, cursor_class=None, connect_timeout=30,
                  autocommit=False, config_overrides_defaults=False, check_hostname=None):
    config = {}

    if config_file and os.path.exists(config_file):
        config['read_default_file'] = config_file

        if config_overrides_defaults:
            try:
                cp = parse_from_mysql_config_file(config_file)
            except Exception as e:
                module.fail_json(msg="Failed to parse %s: %s" % (config_file, to_native(e)))

            # Override some commond defaults with values from config file if needed
            if cp and cp.has_section('client'):
                try:
                    module.params['login_host'] = cp.get('client', 'host', fallback=module.params['login_host'])
                    module.params['login_port'] = cp.getint('client', 'port', fallback=module.params['login_port'])
                except Exception as e:
                    if "got an unexpected keyword argument 'fallback'" in e.message:
                        module.fail_json(msg='To use config_overrides_defaults, '
                                             'it needs Python 3.5+ as the default interpreter on a target host')

    if ssl_ca is not None or ssl_key is not None or ssl_cert is not None or check_hostname is not None:
        config['ssl'] = {}

    if module.params['login_unix_socket']:
        config['unix_socket'] = module.params['login_unix_socket']
    else:
        config['host'] = module.params['login_host']
        config['port'] = module.params['login_port']

    # If login_user or login_password are given, they should override the
    # config file
    if login_user is not None:
        config['user'] = login_user
    if login_password is not None:
        config['password'] = login_password
    if ssl_cert is not None:
        config['ssl']['cert'] = ssl_cert
    if ssl_key is not None:
        config['ssl']['key'] = ssl_key
    if ssl_ca is not None:
        config['ssl']['ca'] = ssl_ca
    if db is not None:
        config['database'] = db
    if connect_timeout is not None:
        config['connect_timeout'] = connect_timeout
    if check_hostname is not None:
        if get_connector_name(mysql_driver) == 'pymysql':
            version_tuple = (n for n in mysql_driver.__version__.split('.') if n != 'None')
            if reduce(lambda x, y: int(x) * 100 + int(y), version_tuple) >= 711:
                config['ssl']['check_hostname'] = check_hostname
            else:
                module.fail_json(msg='To use check_hostname, pymysql >= 0.7.11 is required on the target host')

    if get_connector_name(mysql_driver) == 'pymysql':
        # In case of PyMySQL driver:
        if mysql_driver.version_info[0] < 1:
            # for PyMySQL < 1.0.0, use 'db' instead of 'database' and 'passwd' instead of 'password'
            if 'database' in config:
                config['db'] = config['database']
                del config['database']
            if 'password' in config:
                config['passwd'] = config['password']
                del config['password']
        db_connection = mysql_driver.connect(autocommit=autocommit, **config)
    else:
        # In case of MySQLdb driver

        # Will be deprecated and dropped
        # https://github.com/ansible-collections/ansible.mysql/issues/654
        module.warn('Support of mysqlcline/MySQLdb connector is deprecated. '
                    'We\'ll stop testing against it in collection version 4.0.0 '
                    'and remove the related code in 5.0.0. Use PyMySQL connector instead.')

        if mysql_driver.version_info[0] < 2 or (mysql_driver.version_info[0] == 2 and mysql_driver.version_info[1] < 1):
            # for MySQLdb < 2.1.0, use 'db' instead of 'database' and 'passwd' instead of 'password'
            if 'database' in config:
                config['db'] = config['database']
                del config['database']
            if 'password' in config:
                config['passwd'] = config['password']
                del config['password']
        db_connection = mysql_driver.connect(**config)
        if autocommit:
            db_connection.autocommit(True)

    # Monkey patch the Connection class to close the connection when garbage collected
    def _conn_patch(conn_self):
        conn_self.close()
    db_connection.__class__.__del__ = _conn_patch
    # Patched

    if cursor_class == 'DictCursor':
        return db_connection.cursor(**{_mysql_cursor_param: mysql_driver.cursors.DictCursor}), db_connection
    else:
        return db_connection.cursor(), db_connection


def mysql_common_argument_spec():
    return dict(
        login_user=dict(type='str', default=None),
        login_password=dict(type='str', no_log=True),
        login_host=dict(type='str', default='localhost'),
        login_port=dict(type='int', default=3306),
        login_unix_socket=dict(type='str'),
        config_file=dict(type='path', default='~/.my.cnf'),
        connect_timeout=dict(type='int', default=30),
        client_cert=dict(type='path', aliases=['ssl_cert']),
        client_key=dict(type='path', aliases=['ssl_key']),
        ca_cert=dict(type='path', aliases=['ssl_ca']),
        check_hostname=dict(type='bool', default=None),
    )


def get_server_version(cursor):
    """Returns a string representation of the server version."""
    cursor.execute("SELECT VERSION() AS version")
    result = cursor.fetchone()

    if isinstance(result, dict):
        version_str = result['version']
    else:
        version_str = result[0]

    return version_str


def get_server_implementation(module, cursor):
    if 'mariadb' in get_server_version(cursor).lower():
        # TODO Remove this warning after MariaDB support is dropped
        # https://github.com/ansible-collections/ansible.mysql/milestone/3
        module.warn("MariaDB has been detected: "
                    "its support will be dropped in 6.0.0. "
                    "For MariaDB automation, please use the ansible.mariadb collection instead.")
        return "mariadb"
    else:
        return "mysql"


def set_session_vars(module, cursor, session_vars):
    """Set session vars."""
    for var, value in session_vars.items():
        query = "SET SESSION %s = " % mysql_quote_identifier(var, 'vars')
        try:
            cursor.execute(query + "%s", (value,))
        except Exception as e:
            module.fail_json(msg='Failed to execute %s%s: %s' % (query, value, e))



# Copyright (c) 2021, Andrew Pantuso (@ajpantuso) <ajpantuso@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

from ansible.errors import (
    AnsibleError,
    AnsibleFilterError,
)
from ansible.module_utils.common.collections import is_sequence

try:
    from ansible.errors import AnsibleTypeError
except ImportError:
    from ansible.errors import AnsibleFilterTypeError as AnsibleTypeError  # type: ignore

try:
    from hashids import Hashids

    HAS_HASHIDS = True
except ImportError:
    HAS_HASHIDS = False


def initialize_hashids(**kwargs):
    if not HAS_HASHIDS:
        raise AnsibleError("The hashids library must be installed in order to use this plugin")

    params = {k: v for k, v in kwargs.items() if v}

    try:
        return Hashids(**params)
    except TypeError as e:
        str_params = ", ".join([f"{k}={v}" for k, v in params.items()])
        raise AnsibleFilterError(f"The provided parameters {str_params} are invalid: {e}") from e


def hashids_encode(nums, salt=None, alphabet=None, min_length=None):
    """Generates a YouTube-like hash from a sequence of ints

    :nums: Sequence of one or more ints to hash
    :salt: String to use as salt when hashing
    :alphabet: String of 16 or more unique characters to produce a hash
    :min_length: Minimum length of hash produced
    """

    hashids = initialize_hashids(salt=salt, alphabet=alphabet, min_length=min_length)

    # Handles the case where a single int is not encapsulated in a list or tuple.
    # User convenience seems preferable to strict typing in this case
    # Also avoids obfuscated error messages related to single invalid inputs
    if not is_sequence(nums):
        nums = [nums]

    try:
        hashid = hashids.encode(*nums)
    except TypeError as e:
        raise AnsibleTypeError(f"Data to encode must by a tuple or list of ints: {e}") from e

    return hashid


def hashids_decode(hashid, salt=None, alphabet=None, min_length=None):
    """Decodes a YouTube-like hash to a sequence of ints

    :hashid: Hash string to decode
    :salt: String to use as salt when hashing
    :alphabet: String of 16 or more unique characters to produce a hash
    :min_length: Minimum length of hash produced
    """

    hashids = initialize_hashids(salt=salt, alphabet=alphabet, min_length=min_length)
    nums = hashids.decode(hashid)
    return list(nums)


class FilterModule:
    def filters(self):
        return {
            "hashids_encode": hashids_encode,
            "hashids_decode": hashids_decode,
        }



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The nxos_ntp_global config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.facts import Facts
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.rm_templates.ntp_global import (
    Ntp_globalTemplate,
)


class Ntp_global(ResourceModule):
    """
    The nxos_ntp_global config class
    """

    def __init__(self, module):
        super(Ntp_global, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="ntp_global",
            tmplt=Ntp_globalTemplate(),
        )
        self.parsers = [
            "access_group.match_all",
            "allow.control.rate_limit",
            "allow.private",
            "authenticate",
            "logging",
            "master.stratum",
            "passive",
            "source",
            "source_interface",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = self._ntp_list_to_dict(self.want)
        haved = self._ntp_list_to_dict(self.have)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd
        if self.state == "deleted":
            wantd = {}

        self._compare(want=wantd, have=haved)

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Ntp_global network resource.
        """
        self.compare(parsers=self.parsers, want=want, have=have)
        self._compare_lists(want=want, have=have)
        self._compare_access_group(want=want, have=have)

    def _compare_lists(self, want, have):
        keys = ["authentication_keys", "peers", "servers", "trusted_keys"]
        for x in keys:
            wantx = want.get(x, {})
            havex = have.get(x, {})

            for wkey, wentry in wantx.items():
                hentry = havex.pop(wkey, {})

                # pop aliased keys to preserve idempotence
                if x in ["peers", "servers"]:
                    wentry.pop("use_vrf", None)

                if wentry != hentry:
                    if x in keys[1:3] and self.state in [
                        "overridden",
                        "replaced",
                    ]:
                        # remove existing config else it gets appeneded
                        self.addcmd(hentry, x, negate=True)
                    self.addcmd(wentry, x)

            # remove superfluos config
            for _hkey, hentry in havex.items():
                self.addcmd(hentry, x, negate=True)

    def _compare_access_group(self, want, have):
        want_ag = want.get("access_group", {})
        have_ag = have.get("access_group", {})

        for x in ["peer", "query_only", "serve", "serve_only"]:
            wx = want_ag.get(x, {})
            hx = have_ag.get(x, {})

            for wkey, wentry in wx.items():
                hentry = hx.pop(wkey, {})
                if wentry != hentry:
                    self.addcmd(wentry, x)

            # remove superfluos config
            for hentry in hx.values():
                self.addcmd(hentry, x, negate=True)

    def _ntp_list_to_dict(self, data):
        """Convert all list to dicts to dicts
        of dicts
        """
        tmp = deepcopy(data)
        if "access_group" in tmp:
            for x in ["peer", "query_only", "serve", "serve_only"]:
                if x in tmp["access_group"]:
                    tmp["access_group"][x] = {i["access_list"]: i for i in tmp["access_group"][x]}
        pkey = {
            "authentication_keys": "id",
            "peers": "peer",
            "servers": "server",
            "trusted_keys": "key_id",
        }
        for k in pkey.keys():
            if k in tmp:
                tmp[k] = {i[pkey[k]]: i for i in tmp[k]}
        return tmp



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ses_identity
version_added: 1.0.0
short_description: Manages SES email and domain identity
description:
    - This module allows the user to manage verified email and domain identity for SES.
    - This covers verifying and removing identities as well as setting up complaint, bounce
      and delivery notification settings.
    - Prior to release 5.0.0 this module was called C(community.aws.aws_ses_identity).
      The usage did not change.
author:
    - Ed Costello (@orthanc)
options:
    identity:
        description:
            - This is the email address or domain to verify / delete.
            - If this contains an '@' then it will be considered an email. Otherwise it will be considered a domain.
        required: true
        type: str
    state:
        description: Whether to create(or update) or delete the identity.
        default: present
        choices: [ 'present', 'absent' ]
        type: str
    bounce_notifications:
        description:
            - Setup the SNS topic used to report bounce notifications.
            - If omitted, bounce notifications will not be delivered to a SNS topic.
            - If bounce notifications are not delivered to a SNS topic, I(feedback_forwarding) must be enabled.
        suboptions:
            topic:
                description:
                    - The ARN of the topic to send notifications to.
                    - If omitted, notifications will not be delivered to a SNS topic.
            include_headers:
                description:
                    - Whether or not to include headers when delivering to the SNS topic.
                    - If I(topic) is not specified this will have no impact, but the SES setting is updated even if there is no topic.
                type: bool
                default: No
        type: dict
    complaint_notifications:
        description:
            - Setup the SNS topic used to report complaint notifications.
            - If omitted, complaint notifications will not be delivered to a SNS topic.
            - If complaint notifications are not delivered to a SNS topic, I(feedback_forwarding) must be enabled.
        suboptions:
            topic:
                description:
                    - The ARN of the topic to send notifications to.
                    - If omitted, notifications will not be delivered to a SNS topic.
            include_headers:
                description:
                    - Whether or not to include headers when delivering to the SNS topic.
                    - If I(topic) is not specified this will have no impact, but the SES setting is updated even if there is no topic.
                type: bool
                default: No
        type: dict
    delivery_notifications:
        description:
            - Setup the SNS topic used to report delivery notifications.
            - If omitted, delivery notifications will not be delivered to a SNS topic.
        suboptions:
            topic:
                description:
                    - The ARN of the topic to send notifications to.
                    - If omitted, notifications will not be delivered to a SNS topic.
            include_headers:
                description:
                    - Whether or not to include headers when delivering to the SNS topic.
                    - If I(topic) is not specified this will have no impact, but the SES setting is updated even if there is no topic.
                type: bool
                default: No
        type: dict
    feedback_forwarding:
        description:
            - Whether or not to enable feedback forwarding.
            - This can only be false if both I(bounce_notifications) and I(complaint_notifications) specify SNS topics.
        type: 'bool'
        default: true
extends_documentation_fragment:
    - amazon.aws.common.modules
    - amazon.aws.region.modules
    - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Ensure example@example.com email identity exists
  community.aws.ses_identity:
    identity: example@example.com
    state: present

- name: Delete example@example.com email identity
  community.aws.ses_identity:
    email: example@example.com
    state: absent

- name: Ensure example.com domain identity exists
  community.aws.ses_identity:
    identity: example.com
    state: present

# Create an SNS topic and send bounce and complaint notifications to it
# instead of emailing the identity owner
- name: Ensure complaints-topic exists
  community.aws.sns_topic:
    name: "complaints-topic"
    state: present
    purge_subscriptions: false
  register: topic_info

- name: Deliver feedback to topic instead of owner email
  community.aws.ses_identity:
    identity: example@example.com
    state: present
    complaint_notifications:
      topic: "{{ topic_info.sns_arn }}"
      include_headers: true
    bounce_notifications:
      topic: "{{ topic_info.sns_arn }}"
      include_headers: false
    feedback_forwarding: false

# Create an SNS topic for delivery notifications and leave complaints
# Being forwarded to the identity owner email
- name: Ensure delivery-notifications-topic exists
  community.aws.sns_topic:
    name: "delivery-notifications-topic"
    state: present
    purge_subscriptions: false
  register: topic_info

- name: Delivery notifications to topic
  community.aws.ses_identity:
    identity: example@example.com
    state: present
    delivery_notifications:
      topic: "{{ topic_info.sns_arn }}"
"""

RETURN = r"""
identity:
    description: The identity being modified.
    returned: success
    type: str
    sample: example@example.com
identity_arn:
    description: The arn of the identity being modified.
    returned: success
    type: str
    sample: arn:aws:ses:us-east-1:12345678:identity/example@example.com
verification_attributes:
    description: The verification information for the identity.
    returned: success
    type: complex
    sample: {
        "verification_status": "Pending",
        "verification_token": "...."
    }
    contains:
        verification_status:
            description: The verification status of the identity.
            type: str
            sample: "Pending"
        verification_token:
            description: The verification token for a domain identity.
            type: str
notification_attributes:
    description: The notification setup for the identity.
    returned: success
    type: complex
    sample: {
        "bounce_topic": "arn:aws:sns:....",
        "complaint_topic": "arn:aws:sns:....",
        "delivery_topic": "arn:aws:sns:....",
        "forwarding_enabled": false,
        "headers_in_bounce_notifications_enabled": true,
        "headers_in_complaint_notifications_enabled": true,
        "headers_in_delivery_notifications_enabled": true
    }
    contains:
        bounce_topic:
            description:
              - The ARN of the topic bounce notifications are delivered to.
              - Omitted if bounce notifications are not delivered to a topic.
            type: str
        complaint_topic:
            description:
              - The ARN of the topic complaint notifications are delivered to.
              - Omitted if complaint notifications are not delivered to a topic.
            type: str
        delivery_topic:
            description:
              - The ARN of the topic delivery notifications are delivered to.
              - Omitted if delivery notifications are not delivered to a topic.
            type: str
        forwarding_enabled:
            description: Whether or not feedback forwarding is enabled.
            type: bool
        headers_in_bounce_notifications_enabled:
            description: Whether or not headers are included in messages delivered to the bounce topic.
            type: bool
        headers_in_complaint_notifications_enabled:
            description: Whether or not headers are included in messages delivered to the complaint topic.
            type: bool
        headers_in_delivery_notifications_enabled:
            description: Whether or not headers are included in messages delivered to the delivery topic.
            type: bool
"""

import time

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def get_verification_attributes(connection, module, identity, retries=0, retryDelay=10):
    # Unpredictably get_identity_verification_attributes doesn't include the identity even when we've
    # just registered it. Suspect this is an eventual consistency issue on AWS side.
    # Don't want this complexity exposed users of the module as they'd have to retry to ensure
    # a consistent return from the module.
    # To avoid this we have an internal retry that we use only after registering the identity.
    for attempt in range(0, retries + 1):
        try:
            response = connection.get_identity_verification_attributes(Identities=[identity], aws_retry=True)
        except (BotoCoreError, ClientError) as e:
            module.fail_json_aws(e, msg=f"Failed to retrieve identity verification attributes for {identity}")
        identity_verification = response["VerificationAttributes"]
        if identity in identity_verification:
            break
        time.sleep(retryDelay)
    if identity not in identity_verification:
        return None
    return identity_verification[identity]


def get_identity_notifications(connection, module, identity, retries=0, retryDelay=10):
    # Unpredictably get_identity_notifications doesn't include the notifications when we've
    # just registered the identity.
    # Don't want this complexity exposed users of the module as they'd have to retry to ensure
    # a consistent return from the module.
    # To avoid this we have an internal retry that we use only when getting the current notification
    # status for return.
    for attempt in range(0, retries + 1):
        try:
            response = connection.get_identity_notification_attributes(Identities=[identity], aws_retry=True)
        except (BotoCoreError, ClientError) as e:
            module.fail_json_aws(e, msg=f"Failed to retrieve identity notification attributes for {identity}")
        notification_attributes = response["NotificationAttributes"]

        # No clear AWS docs on when this happens, but it appears sometimes identities are not included in
        # in the notification attributes when the identity is first registered. Suspect that this is caused by
        # eventual consistency within the AWS services. It's been observed in builds so we need to handle it.
        #
        # When this occurs, just return None and we'll assume no identity notification settings have been changed
        # from the default which is reasonable if this is just eventual consistency on creation.
        # See: https://github.com/ansible/ansible/issues/36065
        if identity in notification_attributes:
            break

        # Paranoia check for coding errors, we only requested one identity, so if we get a different one
        # something has gone very wrong.
        if len(notification_attributes) != 0:
            module.fail_json(
                msg="Unexpected identity found in notification attributes, expected {0} but got {1!r}.".format(
                    identity,
                    notification_attributes.keys(),
                )
            )

        time.sleep(retryDelay)
    if identity not in notification_attributes:
        return None
    return notification_attributes[identity]


def desired_topic(module, notification_type):
    arg_dict = module.params.get(notification_type.lower() + "_notifications")
    if arg_dict:
        return arg_dict.get("topic", None)
    else:
        return None


def update_notification_topic(connection, module, identity, identity_notifications, notification_type):
    # Not passing the parameter should not cause any changes.
    if module.params.get(f"{notification_type.lower()}_notifications") is None:
        return False

    topic_key = notification_type + "Topic"
    if identity_notifications is None:
        # If there is no configuration for notifications cannot be being sent to topics
        # hence assume None as the current state.
        current_topic = None
    elif topic_key in identity_notifications:
        current_topic = identity_notifications[topic_key]
    else:
        # If there is information on the notifications setup but no information on the
        # particular notification topic it's pretty safe to assume there's no topic for
        # this notification. AWS API docs suggest this information will always be
        # included but best to be defensive
        current_topic = None

    required_topic = desired_topic(module, notification_type)

    if current_topic != required_topic:
        try:
            if not module.check_mode:
                request_kwargs = {
                    "Identity": identity,
                    "NotificationType": notification_type,
                    "aws_retry": True,
                }

                # The topic has to be omitted from the request to disable the notification.
                if required_topic is not None:
                    request_kwargs["SnsTopic"] = required_topic

                connection.set_identity_notification_topic(**request_kwargs)
        except (BotoCoreError, ClientError) as e:
            module.fail_json_aws(
                e,
                msg=f"Failed to set identity notification topic for {identity} {notification_type}",
            )
        return True
    return False


def update_notification_topic_headers(connection, module, identity, identity_notifications, notification_type):
    arg_dict = module.params.get(notification_type.lower() + "_notifications")
    header_key = "HeadersIn" + notification_type + "NotificationsEnabled"
    if identity_notifications is None:
        # If there is no configuration for topic notifications, headers cannot be being
        # forwarded, hence assume false.
        current = False
    elif header_key in identity_notifications:
        current = identity_notifications[header_key]
    else:
        # AWS API doc indicates that the headers in fields are optional. Unfortunately
        # it's not clear on what this means. But it's a pretty safe assumption that it means
        # headers are not included since most API consumers would interpret absence as false.
        current = False

    if arg_dict is not None and "include_headers" in arg_dict:
        required = arg_dict["include_headers"]
    else:
        required = False

    if current != required:
        try:
            if not module.check_mode:
                connection.set_identity_headers_in_notifications_enabled(
                    Identity=identity, NotificationType=notification_type, Enabled=required, aws_retry=True
                )
        except (BotoCoreError, ClientError) as e:
            module.fail_json_aws(
                e, msg=f"Failed to set identity headers in notification for {identity} {notification_type}"
            )
        return True
    return False


def update_feedback_forwarding(connection, module, identity, identity_notifications):
    if identity_notifications is None:
        # AWS requires feedback forwarding to be enabled unless bounces and complaints
        # are being handled by SNS topics. So in the absence of identity_notifications
        # information existing feedback forwarding must be on.
        current = True
    elif "ForwardingEnabled" in identity_notifications:
        current = identity_notifications["ForwardingEnabled"]
    else:
        # If there is information on the notifications setup but no information on the
        # forwarding state it's pretty safe to assume forwarding is off. AWS API docs
        # suggest this information will always be included but best to be defensive
        current = False

    required = module.params.get("feedback_forwarding")

    if current != required:
        try:
            if not module.check_mode:
                connection.set_identity_feedback_forwarding_enabled(
                    Identity=identity, ForwardingEnabled=required, aws_retry=True
                )
        except (BotoCoreError, ClientError) as e:
            module.fail_json_aws(e, msg=f"Failed to set identity feedback forwarding for {identity}")
        return True
    return False


def create_mock_notifications_response(module):
    resp = {
        "ForwardingEnabled": module.params.get("feedback_forwarding"),
    }
    for notification_type in ("Bounce", "Complaint", "Delivery"):
        arg_dict = module.params.get(notification_type.lower() + "_notifications")
        if arg_dict is not None and "topic" in arg_dict:
            resp[notification_type + "Topic"] = arg_dict["topic"]

        header_key = "HeadersIn" + notification_type + "NotificationsEnabled"
        if arg_dict is not None and "include_headers" in arg_dict:
            resp[header_key] = arg_dict["include_headers"]
        else:
            resp[header_key] = False
    return resp


def update_identity_notifications(connection, module):
    identity = module.params.get("identity")
    changed = False
    identity_notifications = get_identity_notifications(connection, module, identity)

    for notification_type in ("Bounce", "Complaint", "Delivery"):
        changed |= update_notification_topic(connection, module, identity, identity_notifications, notification_type)
        changed |= update_notification_topic_headers(
            connection, module, identity, identity_notifications, notification_type
        )

    changed |= update_feedback_forwarding(connection, module, identity, identity_notifications)

    if changed or identity_notifications is None:
        if module.check_mode:
            identity_notifications = create_mock_notifications_response(module)
        else:
            identity_notifications = get_identity_notifications(connection, module, identity, retries=4)
    return changed, identity_notifications


def validate_params_for_identity_present(module):
    if module.params.get("feedback_forwarding") is False:
        if not (desired_topic(module, "Bounce") and desired_topic(module, "Complaint")):
            module.fail_json(
                msg=(
                    "Invalid Parameter Value 'False' for 'feedback_forwarding'. AWS requires "
                    "feedback forwarding to be enabled unless bounces and complaints are handled by SNS topics"
                )
            )


def create_or_update_identity(connection, module, region, account_id):
    identity = module.params.get("identity")
    changed = False
    verification_attributes = get_verification_attributes(connection, module, identity)
    if verification_attributes is None:
        try:
            if not module.check_mode:
                if "@" in identity:
                    connection.verify_email_identity(EmailAddress=identity, aws_retry=True)
                else:
                    connection.verify_domain_identity(Domain=identity, aws_retry=True)
        except (BotoCoreError, ClientError) as e:
            module.fail_json_aws(e, msg=f"Failed to verify identity {identity}")
        if module.check_mode:
            verification_attributes = {
                "VerificationStatus": "Pending",
            }
        else:
            verification_attributes = get_verification_attributes(connection, module, identity, retries=4)
        changed = True
    elif verification_attributes["VerificationStatus"] not in ("Pending", "Success"):
        module.fail_json(
            msg="Identity " + identity + " in bad status " + verification_attributes["VerificationStatus"],
            verification_attributes=camel_dict_to_snake_dict(verification_attributes),
        )

    if verification_attributes is None:
        module.fail_json(msg="Unable to load identity verification attributes after registering identity.")

    notifications_changed, notification_attributes = update_identity_notifications(connection, module)
    changed |= notifications_changed

    if notification_attributes is None:
        module.fail_json(msg="Unable to load identity notification attributes.")

    identity_arn = "arn:aws:ses:" + region + ":" + account_id + ":identity/" + identity

    module.exit_json(
        changed=changed,
        identity=identity,
        identity_arn=identity_arn,
        verification_attributes=camel_dict_to_snake_dict(verification_attributes),
        notification_attributes=camel_dict_to_snake_dict(notification_attributes),
    )


def destroy_identity(connection, module):
    identity = module.params.get("identity")
    changed = False
    verification_attributes = get_verification_attributes(connection, module, identity)
    if verification_attributes is not None:
        try:
            if not module.check_mode:
                connection.delete_identity(Identity=identity, aws_retry=True)
        except (BotoCoreError, ClientError) as e:
            module.fail_json_aws(e, msg=f"Failed to delete identity {identity}")
        changed = True

    module.exit_json(
        changed=changed,
        identity=identity,
    )


def get_account_id(module):
    sts = module.client("sts")
    try:
        caller_identity = sts.get_caller_identity()
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e, msg="Failed to retrieve caller identity")
    return caller_identity["Account"]


def main():
    module = AnsibleAWSModule(
        argument_spec={
            "identity": dict(required=True, type="str"),
            "state": dict(default="present", choices=["present", "absent"]),
            "bounce_notifications": dict(type="dict"),
            "complaint_notifications": dict(type="dict"),
            "delivery_notifications": dict(type="dict"),
            "feedback_forwarding": dict(default=True, type="bool"),
        },
        supports_check_mode=True,
    )

    for notification_type in ("bounce", "complaint", "delivery"):
        param_name = notification_type + "_notifications"
        arg_dict = module.params.get(param_name)
        if arg_dict:
            extra_keys = [x for x in arg_dict.keys() if x not in ("topic", "include_headers")]
            if extra_keys:
                module.fail_json(
                    msg="Unexpected keys "
                    + str(extra_keys)
                    + " in "
                    + param_name
                    + " valid keys are topic or include_headers"
                )

    # SES APIs seem to have a much lower throttling threshold than most of the rest of the AWS APIs.
    # Docs say 1 call per second. This shouldn't actually be a big problem for normal usage, but
    # the ansible build runs multiple instances of the test in parallel that's caused throttling
    # failures so apply a jittered backoff to call SES calls.
    connection = module.client("ses", retry_decorator=AWSRetry.jittered_backoff())

    state = module.params.get("state")

    if state == "present":
        region = module.region
        account_id = get_account_id(module)
        validate_params_for_identity_present(module)
        create_or_update_identity(connection, module, region, account_id)
    else:
        destroy_identity(connection, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2026 Klaas Weyermann (@Klaas-)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_arcmachineextensions_info
version_added: "3.17.0"
short_description: Get arc machine extensions
description:
    - Get arc machine extensions

options:
    name:
        description:
            - The name of the arc machine extension you're trying to get details about.
        type: str
    machine_name:
        description:
            - The name of the arc machine
        required: true
        type: str
    resource_group:
        description:
            - The name of the resource group in which the arc machine is
        required: true
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Klaas Weyermann (@Klaas-)
'''

EXAMPLES = '''
- name: Get arc machine extension details
  azure.azcollection.azure_rm_arcmachineextensions_info:
    name: extension_name
    machine_name: arc_machine_name
    resource_group: resource_group_name

- name: Get all arc machine extensions of specific arc machine
  azure.azcollection.azure_rm_arcmachineextensions_info:
    machine_name: arc_machine_name
    resource_group: resource_group_name
'''

RETURN = '''
arcmachineextensions:
    description:
        - List of arc machine extensions
        - Can be empty if listing arc machine extensions or arc machine extension does not exist
    type: list
    returned: always
    sample: [
        {
            "id": \
"/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/Resource_Group/providers/Microsoft.HybridCompute/machines/VMName/extensions/AzureMonitorLinuxAgent",
            "location": "germanywestcentral",
            "name": "AzureMonitorLinuxAgent",
            "properties": {
                "auto_upgrade_minor_version": false,
                "enable_automatic_upgrade": true,
                "instance_view": {
                    "name": "AzureMonitorLinuxAgent",
                    "status": {
                        "code": "0",
                        "level": "Information",
                        "message": "Extension Message: Enable succeeded"
                    },
                    "type": "AzureMonitorLinuxAgent",
                    "type_handler_version": "1.37.0"
                },
                "provisioning_state": "Succeeded",
                "publisher": "Microsoft.Azure.Monitor",
                "settings": {},
                "type": "AzureMonitorLinuxAgent",
                "type_handler_version": "1.37.0"
            },
            "tags": {
                "TagKey": "TagValue"
            },
            "type": "Microsoft.HybridCompute/machines/extensions"
        }
    ]
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError, HttpResponseError

except ImportError:
    # This is handled in azure_rm_common
    pass

AZURE_OBJECT_CLASS = 'hybridcomputemachineextensions'


class AzureRMarcmachineextensionsInfo(AzureRMModuleBase):
    """Information class for an Azure RM arc machine extensions"""

    def __init__(self):
        self.module_arg_spec = dict(
            name=dict(type='str', required=False),
            machine_name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True)
        )

        self.name = None
        self.machine_name = None
        self.resource_group = None
        self.log_path = None
        self.log_mode = None

        self.results = dict(
            changed=False,
            arcmachineextensions=[]
        )

        super(AzureRMarcmachineextensionsInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                              supports_check_mode=True,
                                                              supports_tags=False,
                                                              facts_module=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name:
            result = self.get_arc_machine_extension()
        else:
            result = self.list_arc_machine_extensions()

        self.results['arcmachineextensions'] = result

        return self.results

    def get_arc_machine_extension(self):
        '''
        Gets the properties of the specified arc machine extension.

        :return: List of arc machine extensions
        '''
        self.log("Checking if arc machine extension {0} in resource group {1} is present".format(self.name,
                                                                                                 self.resource_group))

        result = []
        arc_machine_extension = None

        try:
            arc_machine_extension = self.hybrid_compute_management_client.machine_extensions.get(machine_name=self.machine_name,
                                                                                                 resource_group_name=self.resource_group,
                                                                                                 extension_name=self.name)
        except ResourceNotFoundError as ex:
            self.log("Could not find arc machine extension {0} in resource group {1}".format(self.name, self.resource_group))
            return []
        except HttpResponseError as ex:
            if ex.error.code == 'InvalidSubscriptionId':
                self.log("Could not find subscription id")
                return []
            else:
                raise Exception(ex)
        if arc_machine_extension:
            result = [self.serialize_obj(arc_machine_extension, AZURE_OBJECT_CLASS)]

        return result

    def list_arc_machine_extensions(self):
        '''
        Gets the properties of the specified arc machine extensions in resource group or subscription.

        :return: List of arc machine extensions
        '''

        result = []
        arc_machine_extensions = None

        self.log("Checking if the arc machine extensions in resource group {0} and machine name {1} are present".format(self.resource_group,
                                                                                                                        self.machine_name))
        arc_machine_extensions_mgmt_client = self.hybrid_compute_management_client.machine_extensions
        arc_machine_extensions = arc_machine_extensions_mgmt_client.list(resource_group_name=self.resource_group,
                                                                         machine_name=self.machine_name)
        if arc_machine_extensions:
            # it seems the exception is thrown when iterating through arc_machine_extensions, not when setting arc_machine_extensions
            try:
                for item in arc_machine_extensions:
                    result.append(self.serialize_obj(item, AZURE_OBJECT_CLASS))
            except ResourceNotFoundError as ex:
                self.log("Could not find resource group {0}".format(self.resource_group))
            except HttpResponseError as ex:
                if ex.error.code == 'InvalidSubscriptionId':
                    self.log("Could not find subscription id")
                else:
                    raise Exception(ex)

        return result


def main():
    """Main execution"""
    AzureRMarcmachineextensionsInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_import_outbound_inspection_certificate
short_description: Import Outbound Inspection certificate for HTTPS inspection.
description:
  - Import Outbound Inspection certificate for HTTPS inspection.
  - All operations are performed over Web Services API.
  - Available from R81.20 management version.
version_added: "6.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  base64_certificate:
    description:
      - Certificate file encoded in base64.<br/>Valid file format, p12.
    type: str
  base64_password:
    description:
      - Password (encoded in Base64 with padding) for the certificate file.
    type: str
  name:
    description:
      - Object name. Must be unique in the domain.
      - Available from R82 management version.
    type: str
  tags:
    description:
      - Collection of tag identifiers.
      - Available from R82 management version.
    type: list
    elements: str
  is_default:
    description:
      - Is the certificate is the default certificate.
      - Available from R82 management version.
    type: bool
  color:
    description:
      - Color of the object. Should be one of existing colors.
      - Available from R82 management version.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
      - Available from R82 management version.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings. By Setting this parameter to 'true' default outbound certificate can be replaced.
      - Available from R82 management version.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
      - Available from R82 management version.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: import-outbound-inspection-certificate
  cp_mgmt_import_outbound_inspection_certificate:
    base64_certificate:
      "MIIKSAIBAzCCCg4GCSqGSIb3DQEHAaCCCf8Eggn7MIIJ9zCCBI8GCSqGSIb3DQEHBqCCBIAwggR8AgEAMIIEdQYJKoZIhvcNAQcBMBwGCiqGSIb3DQEMAQYwDgQILAfxjBi7DTQCAggAgIIESKgKoClNx4
       yTQr7xfIgSBSDs0It2vVsLubNFJpbQXzJUu2WaPQPbqV3wISpWCa/auLYC9OWpTI89HFt30rVAdWCFVoty7jI6L8HjTYa8fTGyqW7PyfoGyZclmz6totsmeVWc8i7wnl9Hk8NZpLWuixNoSLQUqBoloyZE
       ll3i3/Z+/6mDlYkRmpCMQA2YLQm1yc/3n7Fq6grBJDro0tIIoAwIzgCdoKqIMwlDNA9c0eaHeXsP4k9WfJQbK6AyLTvHbrrNrgUyEDJQI6BCkeQwkBW2zRUHoe7s1DSQ5Rwft4koIaDcGovLES5g1gnXzm
       r4/23+rf4/EZszB0QvlYvZIKLQ8O2ofvZ/HK+59fxlhKEiEkW2yhezDGR9s6hZnzZ8vMutisQJ8MO0m9iKVD5AAtif/32iy5+TVIQfqgER+DYVGOuk15YF2VcZGRlQ8pSvBXIkMMUDRqjFxQfKYIMlyk6R
       SSgmIn+EIA9GfaBmEGy2xJYvw6IkUJ+xoR+SYeLYiMw+HkzI+cCOKF7fKPXlOCVvnESEeKwJ4inSxiI2GQG01aN/GNdsx/EM1Xi2LSHfzhG9URIOhjuJIQZn2Z7f3fpTxpWWCpEEVjcQZhoR0KX0DJ/gIx
       iY8UsbNo58FTq5AwMFY6m8hxlHOorqh0MSE/x8LKq0v7JKIxQwrdkyUlVUqdaGreW5MgRdjqOrxQx53nLPdQelKWbR8Gn4KkwFcYCAB1VAe944zqq6YKL4mvNwxk5wyqDjn5UZtPokKFfqBOwOSAGsaZ38
       x/2tqXEgPhWVGFPJlsIUUKBRVTtqxsb2LdaCPHjO8bQhhgOIMEav+iWZAJYudZuolr8Aviccorg1w0sr2eklHbO6yMWrDrvlCVpSawRnLIeeWe+4rwV7SNdcA5hSombTWKRcR8mOkTGjpByiz6+g+3mHOe
       byTrmIfUSENMZy5oYjQfDyNLi0RMmCPCqMjRSwyAs/CDhzz4wTFLEYbu+fUrm2WZc2vhhxafbVrbZ+FcDcnYomYfp8aSxiIIq8+gxT99Oi3WNqhJ+IZGJODWMYRfpKNwgCab8uJt8TV3SVXVIXW0Y28l4Z
       P/qWEfnEC8Wl6HJGhJo7arqBFTWWEuKvHw985OpksavdQFXgVU9Egbue0anb0U5SDyRu0hqJ/Gw83dKJbCg8hPv4gGq/yeOb+cX63DCKvOcoXjZ0szeRcGiro0+BSgr143Ks19lsxWHPOlauLSnD3jVrgp
       mVwxCizRTnX3OLJ07IpvvEJGAQR/Ru2lo7eN0H4933G93tVQtte69BiPwbkWtSx8ddzbRGmMW7IsG72FVm5QrJC1C1Na5xqQQV6G2oHqIHNdNyXD6TmhuQ4BnpCoamCzfsX4iozS+NySz/Jdbuj0YZ9L2d
       YUHiBF4xotlHfwiAiCghaBH31OZJ0n52d0NGqRkN5F0Qdfz1O2+rLx2zswggVgBgkqhkiG9w0BBwGgggVRBIIFTTCCBUkwggVFBgsqhkiG9w0BDAoBAqCCBO4wggTqMBwGCiqGSIb3DQEMAQMwDgQIRNvl
       6KdajoCAggABIIEyBJbsgafEO1D9xQ8BFYFNKf/meJNAOO4XVPTFtUBpyvEn3PkyxyKU1cMenESXeMacSv/VftkYC7CwN81kzbRMRSEXZSCsyj48kMqwTqMNmZmgF8XaFvzXOGlu2E411LZ/sOenWO7lxe
       NGZM3vk4FWvl+4fa5Xd5TDqya65VsXSocDUA5kpeqn323TcdeCldGmEniX85NGIiPpWuRLGrNf8VOIuE3NFAmTSveHH9Oo7PjscCifc7O4+NpOW9GfayZMqG8dTpLhIRacdvy/QvbWePXdzzSI9rKogX/7
       bSzU0Hq+8rpWlAhz0qnW2Bb3T7of86Len5cuNr0k425Dhpuo4od81exDdSa3+aFQqR3nKVSkPapLBrpGNZIX4TwctRnbi2ZHdFxMKkJewGt/beam3LcujJRlN2RBeA0IRWEAyO6ubjpQ62ChrW+faHXXxY
       H3Be6nPXSF5pq4VAIVglNsPOxGYIb+qNDhOblzQBq4nF30fyHmOwDIRgNWwOStT7dUFmN0ouHinP6QXWBDDQiDo2RRFs2/RWu0ZY0EAzEYAMCSvmk+SQgKbKpNFf0C5kuJ56PWXUuGSoAXV/vxvK6OHIGF
       FcZo+VrRgYTHY/eSjw1+/lpUkwaWAzoH0X6KxuLXfgzv+E8Z+LFVWIAoknJ96ieljiHzNnfeSTZYwTaJbYaritdAQ2MTGcBrpJFIqr9GjWGVsFQK0ct/ZIFzZw0Vnt/aOj5OjMPlpy9UXfC+tw9gfRYWfS
       uDLuUH0Znu3JB/+J2XQP4PBArXKyvFv6wMVSvY/04r2WQQKV9YTUCkbgvHAlQ7vP0a8z44xSrKc4M04sEBE3cFD2NBAQrP3GqRyz2ukuzJhrj/B1dZWA23SZaqfN9gpbfFbtPXN6F/nY1UUsikLjcXDjC8
       GVU9Pp4VCnv2EUgl4QmkUEdVeDZjUnz/k9Kd53q3h+chAId+3VBsemd3ZadX4gupw6Xf6zT8Av7v75/1/vFw2yz22DG8pIpN4uuEdSFhvs9lr6f2M6bQABS+NWfehq5aqBqsXXX8R3fSxYLL0gO4lxf4Yq
       SomA4AlzS9tJtEe2DKWYmnYwiiUGYLs7aGMLZQbHbYutPKKZXTaSGWYBaIrVjbDM67la/csYmxpb2n6UD6TkNICuZwd/ImVvDhbCEsR/EU+YU0HPwxlUtcCsqw4Vy8rBtbla2XmegGUcLWSurKmq42SW8W
       LBJQfY/9sWyaMqSGy0/Vq4/+/CtXUZ1N5rgibYyIZ9Tvm/ndv2xBW1hYivIZZQFRbg5fWxKA5ifYejGmYCWGQynRSVCbqccw08xy5Iwnww4v5Cz5bcNyRLFOU2/bfn7SC5mcQ/Tw5ZKOQVRn88G78amMPH
       RqX4RzPtIwmK+B3zPJX0MHrY3w5hzPZ0UCtR2YsbYLeqsYP6b+RBLSV3wtkUZ9PgbMeu7zXSE0z1svGpjF7yWpnP47ilbxwe1YXL5+CuqN6iHFfyaP1JPYILmHdw0gzgyOdo1y4rUXgCeiCyH4vJVLts8E
       KpXZDMCUmujb306IOD9haFXdQHV5XlQurtw+JC7ySe9bVMrzYJv5/oPioOXMnLPI2OXYbACwlQ/UHgl5LmDlsxeairdfYTdAxajFEMB0GCSqGSIb3DQEJFDEQHg4AbQB5AGEAbABpAGEAczAjBgkqhkiG9
       0BCRUxFgQU7cUIcmKuQKAMfwbKiKzQozUsyHwwMTAhMAkGBSsOAwIaBQAEFEFoI0QTIv2s2lR8PxS8xfiT5S06BAjANT3YLoakoAICCAA="
    base64_password: bXlfcGFzc3dvcmQ=
    is_default: 'false'
    name: OutboundCertificate
"""

RETURN = """
cp_mgmt_import_outbound_inspection_certificate:
  description: The checkpoint import-outbound-inspection-certificate output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        base64_certificate=dict(type='str'),
        base64_password=dict(type='str', no_log=True),
        name=dict(type='str'),
        tags=dict(type='list', elements='str'),
        is_default=dict(type='bool'),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "import-outbound-inspection-certificate"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: (c) 2023, Gaudenz Steinlin <gaudenz.steinlin@cloudscale.ch>
# Copyright: (c) 2023, Kenneth Joss <kenneth.joss@cloudscale.ch>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: load_balancer_pool
short_description: Manages load balancer pools on the cloudscale.ch IaaS service
description:
  - Get, create, update, delete pools on the cloudscale.ch IaaS service.
notes:
  - If I(uuid) option is provided, it takes precedence over I(name) for pool selection. This allows to update the load balancer pool's name.
  - If no I(uuid) option is provided, I(name) is used for pool selection. If more than one pool with this name exists, execution is aborted.
author:
  - Gaudenz Steinlin (@gaudenz)
  - Kenneth Joss (@k-304)
version_added: "2.3.0"
options:
  state:
    description:
      - State of the load balancer pool.
    choices: [ present, absent ]
    default: present
    type: str
  name:
    description:
      - Name of the load balancer pool.
    type: str
  uuid:
    description:
      - UUID of the load balancer pool.
      - Either I(name) or I(uuid) are required.
    type: str
  load_balancer:
    description:
      - UUID of the load balancer for this pool.
    type: str
  algorithm:
    description:
      - The algorithm according to which the incoming traffic is distributed between the pool members.
      - See the [API documentation](https://www.cloudscale.ch/en/api/v1#pool-algorithms) for supported distribution algorithms.
    type: str
  protocol:
    description:
      - The protocol used for traffic between the load balancer and the pool members.
      - See the [API documentation](https://www.cloudscale.ch/en/api/v1#pool-protocols) for supported protocols.
    type: str
  tags:
    description:
      - Tags assosiated with the load balancer. Set this to C({}) to clear any tags.
    type: dict
extends_documentation_fragment: cloudscale_ch.cloud.api_parameters
'''

EXAMPLES = '''
# Create a pool for a load balancer using registered variables
- name: Create a running load balancer
  cloudscale_ch.cloud.load_balancer:
    name: 'lb1'
    flavor: 'lb-standard'
    zone: 'lpg1'
    tags:
      project: ansible-test
      stage: production
      sla: 24-7
    api_token: xxxxxx
  register: load_balancer

- name: Create a load balancer pool
  cloudscale_ch.cloud.load_balancer_pool:
    name: 'swimming-pool'
    load_balancer: '{{ load_balancer.uuid }}'
    algorithm: 'round_robin'
    protocol: 'tcp'
    tags:
      project: ansible-test
      stage: production
      sla: 24-7
    api_token: xxxxxx
  register: load_balancer_pool

# Create a load balancer pool with algorithm: round_robin and protocol: tcp
- name: Create a load balancer pool
  cloudscale_ch.cloud.load_balancer_pool:
    name: 'cloudscale-loadbalancer-pool1'
    load_balancer: '3766c579-3012-4a85-8192-2bbb4ef85b5f'
    algorithm: 'round_robin'
    protocol: 'tcp'
    tags:
      project: ansible-test
      stage: production
      sla: 24-7
    api_token: xxxxxx

# Get load balancer pool facts by name
- name: Get facts of a load balancer pool
  cloudscale_ch.cloud.load_balancer_pool:
    name: cloudscale-loadbalancer-pool1
    api_token: xxxxxx
'''

RETURN = '''
href:
  description: API URL to get details about this load balancer
  returned: success when not state == absent
  type: str
  sample: https://api.cloudscale.ch/v1/load-balancers/pools/
uuid:
  description: The unique identifier for this load balancer pool
  returned: success
  type: str
  sample: 3766c579-3012-4a85-8192-2bbb4ef85b5f
name:
  description: The display name of the load balancer pool
  returned: success
  type: str
  sample: web-lb-pool1
created_at:
  description: The creation date and time of the load balancer pool
  returned: success when not state == absent
  type: str
  sample: "2023-02-07T15:32:02.308041Z"
load_balancer:
  description: The load balancer this pool is connected to
  returned: success when not state == absent
  type: list
  sample: {
            "href": "https://api.cloudscale.ch/v1/load-balancers/15264769-ac69-4809-a8e4-4d73f8f92496",
            "uuid": "15264769-ac69-4809-a8e4-4d73f8f92496",
            "name": "web-lb"
          }
algorithm:
  description: The algorithm according to which the incoming traffic is distributed between the pool members
  returned: success
  type: str
  sample: round_robin
protocol:
  description: The protocol used for traffic between the load balancer and the pool members
  returned: success
  type: str
  sample: tcp
state:
  description: The current state of the load balancer pool
  returned: success
  type: str
  sample: present
tags:
  description: Tags assosiated with the load balancer
  returned: success
  type: dict
  sample: { 'project': 'my project' }
'''

from ansible.module_utils.basic import AnsibleModule
from ..module_utils.api import (
    AnsibleCloudscaleBase,
    cloudscale_argument_spec,
)

ALLOWED_STATES = ('present',
                  'absent',
                  )


def main():
    argument_spec = cloudscale_argument_spec()
    argument_spec.update(dict(
        name=dict(),
        uuid=dict(),
        load_balancer=dict(),
        algorithm=dict(type='str'),
        protocol=dict(type='str'),
        tags=dict(type='dict'),
        state=dict(default='present', choices=ALLOWED_STATES),
    ))

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=(),
        required_one_of=(('name', 'uuid'),),
        required_if=(('state', 'present', ('name',),),),
        supports_check_mode=True,
    )

    cloudscale_load_balancer_pool = AnsibleCloudscaleBase(
        module,
        resource_name='load-balancers/pools',
        resource_create_param_keys=[
            'name',
            'load_balancer',
            'algorithm',
            'protocol',
            'tags',
        ],
        resource_update_param_keys=[
            'name',
            'tags',
        ],
    )
    cloudscale_load_balancer_pool.query_constraint_keys = []

    if module.params['state'] == "absent":
        result = cloudscale_load_balancer_pool.absent()
    else:
        result = cloudscale_load_balancer_pool.present()
    module.exit_json(**result)


if __name__ == '__main__':
    main()



# Copyright (c) Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: sumologic
type: notification
short_description: Sends task result events to Sumologic
author: "Ryan Currah (@ryancurrah)"
description:
  - This callback plugin sends task results as JSON formatted events to a Sumologic HTTP collector source.
requirements:
  - Whitelisting this callback plugin
  - 'Create a HTTP collector source in Sumologic and specify a custom timestamp format of V(yyyy-MM-dd HH:mm:ss ZZZZ) and
    a custom timestamp locator of V("timestamp": "(.*\)")'
options:
  url:
    description: URL to the Sumologic HTTP collector source.
    type: str
    env:
      - name: SUMOLOGIC_URL
    ini:
      - section: callback_sumologic
        key: url
"""

EXAMPLES = r"""
examples: |-
  To enable, add this to your ansible.cfg file in the defaults block
    [defaults]
    callback_whitelist = community.general.sumologic

  Set the environment variable
    export SUMOLOGIC_URL=https://endpoint1.collection.us2.sumologic.com/receiver/v1/http/R8moSv1d8EW9LAUFZJ6dbxCFxwLH6kfCdcBfddlfxCbLuL-BN5twcTpMk__pYy_cDmp==

  Set the ansible.cfg variable in the callback_sumologic block
    [callback_sumologic]
    url = https://endpoint1.collection.us2.sumologic.com/receiver/v1/http/R8moSv1d8EW9LAUFZJ6dbxCFxwLH6kfCdcBfddlfxCbLuL-BN5twcTpMk__pYy_cDmp==
"""

import getpass
import json
import socket
import uuid
from os.path import basename

from ansible.module_utils.ansible_release import __version__ as ansible_version
from ansible.module_utils.urls import open_url
from ansible.parsing.ajson import AnsibleJSONEncoder
from ansible.plugins.callback import CallbackBase

from ansible_collections.community.general.plugins.module_utils._datetime import (
    now,
)


class SumologicHTTPCollectorSource:
    def __init__(self):
        self.ansible_check_mode = False
        self.ansible_playbook = ""
        self.session = str(uuid.uuid4())
        self.host = socket.gethostname()
        self.ip_address = socket.gethostbyname(socket.gethostname())
        self.user = getpass.getuser()

    def send_event(self, url, state, result, runtime):
        if result._task_fields["args"].get("_ansible_check_mode") is True:
            self.ansible_check_mode = True

        if result._task._role:
            ansible_role = str(result._task._role)
        else:
            ansible_role = None

        if "args" in result._task_fields:
            del result._task_fields["args"]

        data = {}
        data["uuid"] = result._task._uuid
        data["session"] = self.session
        data["status"] = state
        data["timestamp"] = now().strftime("%Y-%m-%d %H:%M:%S +0000")
        data["host"] = self.host
        data["ip_address"] = self.ip_address
        data["user"] = self.user
        data["runtime"] = runtime
        data["ansible_version"] = ansible_version
        data["ansible_check_mode"] = self.ansible_check_mode
        data["ansible_host"] = result._host.name
        data["ansible_playbook"] = self.ansible_playbook
        data["ansible_role"] = ansible_role
        data["ansible_task"] = result._task_fields
        data["ansible_result"] = result._result

        open_url(
            url,
            data=json.dumps(data, cls=AnsibleJSONEncoder, sort_keys=True),
            headers={"Content-type": "application/json", "X-Sumo-Host": data["ansible_host"]},
            method="POST",
        )


class CallbackModule(CallbackBase):
    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = "notification"
    CALLBACK_NAME = "community.general.sumologic"
    CALLBACK_NEEDS_WHITELIST = True

    def __init__(self, display=None):
        super().__init__(display=display)
        self.start_datetimes = {}  # Collect task start times
        self.url = None
        self.sumologic = SumologicHTTPCollectorSource()

    def _runtime(self, result):
        return (now() - self.start_datetimes[result._task._uuid]).total_seconds()

    def set_options(self, task_keys=None, var_options=None, direct=None):
        super().set_options(task_keys=task_keys, var_options=var_options, direct=direct)

        self.url = self.get_option("url")

        if self.url is None:
            self.disabled = True
            self._display.warning(
                "Sumologic HTTP collector source URL was "
                "not provided. The Sumologic HTTP collector "
                "source URL can be provided using the "
                "`SUMOLOGIC_URL` environment variable or "
                "in the ansible.cfg file."
            )

    def v2_playbook_on_start(self, playbook):
        self.sumologic.ansible_playbook = basename(playbook._file_name)

    def v2_playbook_on_task_start(self, task, is_conditional):
        self.start_datetimes[task._uuid] = now()

    def v2_playbook_on_handler_task_start(self, task):
        self.start_datetimes[task._uuid] = now()

    def v2_runner_on_ok(self, result, **kwargs):
        self.sumologic.send_event(self.url, "OK", result, self._runtime(result))

    def v2_runner_on_skipped(self, result, **kwargs):
        self.sumologic.send_event(self.url, "SKIPPED", result, self._runtime(result))

    def v2_runner_on_failed(self, result, **kwargs):
        self.sumologic.send_event(self.url, "FAILED", result, self._runtime(result))

    def runner_on_async_failed(self, result, **kwargs):
        self.sumologic.send_event(self.url, "FAILED", result, self._runtime(result))

    def v2_runner_on_unreachable(self, result, **kwargs):
        self.sumologic.send_event(self.url, "UNREACHABLE", result, self._runtime(result))
